// File.cpp: implementation of the File class.
//
//////////////////////////////////////////////////////////////////////

#include "File.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

File::File() {
	m_pszFileName = NULL;
	m_pszData = NULL;
	m_bInMemory = false;
	m_pszMimeType = NULL;
	m_pszLastModifiedTime = NULL;
	m_iLength = 0;
	m_iLastModifiedTime = -1;
	m_iRef = 0;
	m_pmMutex = new Mutex();
}

File::~File() {

	if (m_pszFileName != NULL) {
		delete [] m_pszFileName;
	}
	
	if (m_pszData != NULL) {
		delete [] m_pszData;
	}

	if (m_pszMimeType != NULL) {
		delete [] m_pszMimeType;
	}
	
	if (m_pszLastModifiedTime != NULL) {
		delete [] m_pszLastModifiedTime;
	}

	delete m_pmMutex;
}

void File::AddRef() {
	m_pmMutex->WaitForSingleObject();
	m_iRef ++;
	m_pmMutex->ReleaseMutex();
}

void File::Release() {
	m_pmMutex->WaitForSingleObject();
	m_iRef --;
	m_pmMutex->ReleaseMutex();
}

int File::GetNumRefs() {
	m_pmMutex->WaitForSingleObject();
	int iRetVal = m_iRef;
	m_pmMutex->ReleaseMutex();
	return iRetVal;
}


int File::GetNumBytes() {
	return m_bInMemory ? m_iLength:0;
}


int File::Open (char* pszFileName) {

	filebuf fbFile;
	fbFile.open (pszFileName, ios::in | ios::nocreate | ios::binary, filebuf::sh_read);

	if (fbFile.is_open() == 0) {
		
		m_pszFileName = NULL;
		m_pszData = NULL;
		m_bInMemory = false;
		m_pszMimeType = NULL;
		m_pszLastModifiedTime = NULL;
		
		return ERR;

	} else {
		
		m_pszFileName = new char [strlen (pszFileName) + 1];
		sprintf (m_pszFileName, pszFileName);
		m_pszMimeType = Utility::GetMimeType (pszFileName);
		
		// Get file data
		struct _stat stFile;
		_stat (pszFileName, &stFile);
		
		// Length
		m_iLength = stFile.st_size;

		// Last modified time int
		m_iLastModifiedTime = stFile.st_mtime;

		// Last modified time string
		char* pszTime = ctime ((const long*) &m_iLastModifiedTime);
		int iLength = strlen (pszTime);
		m_pszLastModifiedTime = new char [iLength + 1];
		sprintf (m_pszLastModifiedTime, pszTime);
		m_pszLastModifiedTime[iLength - 1] = '\0';

		if (m_iLength < MAX_FILE_SIZE_BYTES) {
			m_pszData = new char [m_iLength + 2];
			m_iLength = fbFile.sgetn (m_pszData, m_iLength);	
			m_bInMemory = true;
		} else {
			m_bInMemory = false;
			m_pszData = NULL;
		}
	
		fbFile.close();

		return OK;
	}
}


void File::Close() {

	if (m_pszFileName != NULL) {
		delete [] m_pszFileName;
	}
	
	if (m_pszData != NULL) {
		delete [] m_pszData;
	}

	if (m_pszMimeType != NULL) {
		delete [] m_pszMimeType;
	}

	if (m_pszLastModifiedTime != NULL) {
		m_pszLastModifiedTime;
	}

	m_pszFileName = NULL;
	m_pszData = NULL;
	m_bInMemory = false;
	m_iLength = 0;
	m_iLastModifiedTime = 0;
	m_pszMimeType = NULL;
	m_pszLastModifiedTime = NULL;
}


void File::Delete() {
	::DeleteFile (m_pszFileName);
}

int File::GetLength() const {
	return m_iLength;
}

char* File::GetMimeType() const {
	return m_pszMimeType;
}


void File::SetMimeType (char* pszMimeType) {

	if (pszMimeType != NULL) {
		delete [] m_pszMimeType;
	}

	m_pszMimeType = new char [strlen (pszMimeType) + 1];
	sprintf (m_pszMimeType, pszMimeType);
}


char* File::GetName() const {
	return m_pszFileName;
}

char* File::GetData() const {
	return m_bInMemory ? m_pszData:NULL;
}

char* File::GetLastModifiedTime() const {
	return m_pszLastModifiedTime;
}

int File::GetLastModifiedIntTime() const {
	return m_iLastModifiedTime;
}

int File::ReadData (void** ppState, char* pszBuffer, int iNumBytes) const {

	int iDiff, iRetVal;

	if (m_bInMemory) {

		int* piPos = (int*) *ppState;

		if (piPos == NULL) {
			piPos = new int[1];
			piPos[0] = 0;
			*ppState = (void*) piPos;
		}
			
		iDiff = m_iLength - (*piPos);
		if (iDiff < iNumBytes) {
			
			// EOF
			memcpy (pszBuffer, &m_pszData[piPos[0]], iDiff * sizeof (char));
			iRetVal = iDiff;
			delete [] piPos;
			*ppState = NULL;
		
		} else {
		
			memcpy (pszBuffer, &m_pszData[piPos[0]], iNumBytes * sizeof (char));
			iRetVal = iNumBytes;
			piPos[0] += iRetVal;
		}

	} else {

		filebuf* pfbFile = (filebuf*) *ppState;
		
		if (pfbFile == NULL) {
			pfbFile = new filebuf();
			pfbFile->open (m_pszFileName, ios::in | ios::nocreate | ios::binary, filebuf::sh_read);
			*ppState = pfbFile;
		}
		
		iRetVal = pfbFile->sgetn (pszBuffer, iNumBytes);
		
		// EOF?
		if (iRetVal < iNumBytes) {
			pfbFile->close();
			delete pfbFile;
		}
	}

	return iRetVal;
}


void File::GetAllFileNames (char* pszSpec, int* piNumFiles, char*** pppszFileName) {

	_finddata_t fdFileData;
	long lHandle = _findfirst (pszSpec, &fdFileData);

	// Get num files
	if (lHandle > -1) {
		*piNumFiles = 1;
	} else {
		*piNumFiles = 0;
		*pppszFileName = NULL;
		return;
	}

	while (_findnext (lHandle, &fdFileData) == 0) {
		(*piNumFiles) ++;
	}

	_findclose (lHandle);

	// Get file names
	int i = 1;
	*pppszFileName = new char* [*piNumFiles];

	lHandle = _findfirst (pszSpec, &fdFileData);
	(*pppszFileName)[0] = new char [strlen (fdFileData.name) + 1];
	strcpy ((*pppszFileName)[0], fdFileData.name);

	while (_findnext (lHandle, &fdFileData) == 0) {
		(*pppszFileName)[i] = new char [strlen (fdFileData.name) + 1];
		strcpy ((*pppszFileName)[i], fdFileData.name);
		i ++;
	}

	_findclose (lHandle);
}