// FileCache.h: interface for the FileCache class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_FILECACHE_H__C5064725_A72E_11D1_9C50_0060083E8062__INCLUDED_)
#define AFX_FILECACHE_H__C5064725_A72E_11D1_9C50_0060083E8062__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "File.h"
#include "HashTable.h"
#include "Thread.h"

class FileCache {

	friend class File;

private:

	int m_iMaxNumBytes;
	int m_iNumBuckets;
	int m_iNumFiles;
	int m_iNumBytes;

	StringLinkedList<File>** m_ppBucket;
	Mutex** m_ppBucketMutex;
	Mutex** m_ppCounterMutex;
	Mutex* m_pStatMutex;
	Mutex* m_pExitMutex;
	int* m_piWriters;
	int* m_piReaders;
	bool m_bExit;
	Thread* m_pSizeControlThread;
	Thread* m_pRefreshControlThread;

	StringLinkedList<File>* m_pReadList;
	Mutex* m_pReadMutex;

	StringLinkedList<File>* m_pFifoList;
	Mutex* m_pFifoMutex;

	virtual inline bool Exit();

	virtual File* GetFileFromCache (char* pszFileName);
	virtual File* RetrieveFile (char* pszFileName);
	virtual void AddFile (char* pszFileName, File* pfFile);
	virtual inline int GetHashValue (char* pszFileName);
	static int SizeControlThread (void* lpvData);
	static int RefreshControlThread (void* lpvData);

public:
	FileCache();
	FileCache (int iMaxNumBytes);
	virtual ~FileCache();
	virtual File* GetFile (char* pszFileName);
	virtual void ReleaseFile (File* pfFile);

};

#endif // !defined(AFX_FILECACHE_H__C5064725_A72E_11D1_9C50_0060083E8062__INCLUDED_)

