// HashTable.h: interface for the HashTable class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HASHTABLE_H__653FDF75_B212_11D1_9C6D_0060083E8062__INCLUDED_)
#define AFX_HASHTABLE_H__653FDF75_B212_11D1_9C6D_0060083E8062__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "Mutex.h"

template <class T> class StringListNode;
template <class T> class StringLinkedList;

template <class T> class HashTable {
protected:
	
	StringLinkedList<T>** m_ppBucket;
	int m_iNumBuckets;
	int m_iNumElements;
	
	int GetHashValue (char* pszName) {
		
		int i, iHash = 0, iLength = strlen (pszName);
		
		if (iLength == 0) {
			return 0;
		}

		char* pszNewName = new char [iLength + 1];
		strcpy (pszNewName, pszName);
		char* pszLowerName = _strlwr (pszNewName);

		for (i = 0; i < iLength; i ++) {
			iHash += (int) pszLowerName[i];
		}

		delete [] pszNewName;

		return iHash % m_iNumBuckets;
	}
	
public:

	HashTable (int iNumBuckets) {
		this->Init (iNumBuckets);
	}

	HashTable() {}

	virtual void Init (int iNumBuckets) {
		m_iNumBuckets = iNumBuckets;
		m_ppBucket = new StringLinkedList<T>* [iNumBuckets];
		
		for (int i = 0; i < iNumBuckets; i ++) {
			m_ppBucket[i] = NULL;
		}
		
		m_iNumElements = 0;
	}

	virtual ~HashTable() {
		if (m_ppBucket != NULL) {
			for (int i = 0; i < m_iNumBuckets; i ++) {
				if (m_ppBucket[i] != NULL) {
					delete m_ppBucket[i];
				}
			}
			
			delete m_ppBucket;
		}
	}
	
	virtual void AddElement (char* pszName, T* ptData) {
		if (pszName != NULL) {
			
			// Get hash value
			int iHash = this->GetHashValue (pszName);
			
			// Add element to appropriate linked list
			if (m_ppBucket[iHash] == NULL) {
				m_ppBucket[iHash] = new StringLinkedList<T>();
			}
			
			m_ppBucket[iHash]->AddElement (pszName, ptData);
			
			m_iNumElements ++;
			
		} else {
			
			_ASSERT (false);
		}
	}
	

	virtual T* GetElement (char* pszName) {
		
		if (pszName == NULL) {
			return NULL;
		}
		
		int iHash = this->GetHashValue (pszName);
		
		return (m_ppBucket[iHash] == NULL) ? NULL : m_ppBucket[iHash]->GetElement (pszName);
	}
	

	virtual void DeleteElement (char* pszName) {
		
		if (pszName != NULL) {
			
			// Get hash value
			int iHash = this->GetHashValue (pszName);
			
			// Add element to appropriate linked list
			if (m_ppBucket[iHash] != NULL) {
				m_ppBucket[iHash]->DeleteElement (pszName);
				m_iNumElements --;

				char* pszTemp;
				T* ptTemp;
				if (!m_ppBucket[iHash]->GetFirstElement (&pszTemp, &ptTemp)) {
					delete m_ppBucket[iHash];
					m_ppBucket[iHash] = NULL;
				}
			}
			
		} else {
			
			_ASSERT (false);
		}
	}
};


// Data structures
template <class T> class StringListNode {
public:
	StringListNode (char* pszName, T* ptData, StringListNode<T>* plnNext) {
		
		Name = new char [strlen (pszName) + 1];
		strcpy (Name, pszName);
		Data = ptData;
		Next = plnNext;
	}

	virtual ~StringListNode() {
		if (Name != NULL) {
			delete [] Name;
		}

		if (Data != NULL) {
			delete Data;
		}

		if (Next != NULL) {
			delete Next;
		}
	}

	char* Name;
	T* Data;
	StringListNode<T>* Next;
};


template <class T> class StringLinkedList {
public:

	StringListNode<T>* m_plnHead;
	StringListNode<T>* m_plnTail;

	StringLinkedList() {
		m_plnHead = NULL;
		m_plnTail = NULL;
	}

	virtual ~StringLinkedList() {
		if (m_plnHead != NULL) {
			delete m_plnHead;
		}
	}

	virtual void AddElement (char* pszName, T* ptData) {

		if (m_plnTail != NULL) {
			m_plnTail->Next = new StringListNode<T> (pszName, ptData, NULL);
			m_plnTail = m_plnTail->Next;
		} else {
			m_plnTail = new StringListNode<T> (pszName, ptData, NULL);
			m_plnHead = m_plnTail;
		}
	}

	virtual void DeleteElement (char* pszName) {

		if (pszName == NULL) {
			return;
		}

		StringListNode<T>* plnTemp = m_plnHead;
		StringListNode<T>* plnPrev = NULL;

		while (plnTemp != NULL) {

			if (stricmp (pszName, plnTemp->Name) == 0) {

				if (plnPrev != NULL) {
					plnPrev->Next = plnTemp->Next;
				} else {
					m_plnHead = plnTemp->Next;
				}

				if (plnTemp == m_plnTail) {
					m_plnTail = plnPrev;
				} else {
					plnTemp->Next = NULL;
				}

				delete plnTemp;

				break;
			}

			plnPrev = plnTemp;
			plnTemp = plnTemp->Next;
		}
	}

	virtual void DeleteFirstElement() {

		if (m_plnHead == NULL) {
			return;
		}

		if (m_plnHead != m_plnTail) {
			StringListNode<T>* plnTemp = m_plnHead->Next;
			m_plnHead->Next = NULL;
			m_plnHead->Data = NULL;
			delete m_plnHead;
			m_plnHead = plnTemp;
		} else {
			m_plnHead->Data = NULL;
			delete m_plnHead;
			m_plnHead = NULL;
			m_plnTail = NULL;
		}
	}

	virtual T* GetElement (char* pszName) {

		if (pszName == NULL) {
			return NULL;
		}

		StringListNode<T>* plnTemp = m_plnHead;

		while (plnTemp != NULL) {

			if (stricmp (pszName, plnTemp->Name) == 0) {
				return plnTemp->Data;
			}

			plnTemp = plnTemp->Next;
		}

		return NULL;
	}

	virtual bool GetFirstElement (char** ppszName, T** pptData) {
		if (m_plnHead == NULL) {
			*ppszName = NULL;
			*pptData = NULL;
			return false;
		} else {
			*ppszName = m_plnHead->Name;
			*pptData = m_plnHead->Data;
			return true;
		}
	}
};

#endif // !defined(AFX_HASHTABLE_H__653FDF75_B212_11D1_9C6D_0060083E8062__INCLUDED_)