// HttpRequest.cpp: implementation of the HttpRequest class.
//
//////////////////////////////////////////////////////////////////////

#include "HttpRequest.h"
#include "PageSource.h"
#include "HashTable.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HttpRequest::HttpRequest() {

	m_iMethod = UNSUPPORTED_HTTP_METHOD;
	m_iVersion = UNSUPPORTED_HTTP_VERSION;

	m_pszUri = NULL;
	m_pszFileName = NULL;
	m_pszBrowserName = NULL;
	m_pszIPAddress = NULL;
	m_pszDomainName = NULL;
	m_pszHostName = NULL;

	m_pszLastModified = NULL;

	m_pszHeaders = NULL;

	m_pszLogin = NULL;
	m_pszPassword = NULL;

	m_iNumForms = 0;
	m_ppszFormName = NULL;
	m_ppszFormValue = NULL;
	m_piFormType = NULL;
	m_ppszFormFileName = NULL;

	m_ppszCookieName = NULL;
	m_ppszCookieValue = NULL;

	m_pPageSource = NULL;

	m_bKeepAlive = false;
	m_bCached = false;

	m_iStatus = ERR;
}

HttpRequest::~HttpRequest() {
	
	int i;

	if (m_pszUri != NULL) {
		delete [] m_pszUri;
	}

	if (m_pszFileName != NULL) {
		delete [] m_pszFileName;
	}

	if (m_pszBrowserName != NULL) {
		delete [] m_pszBrowserName;
	}

	if (m_pszIPAddress != NULL) {
		delete [] m_pszIPAddress;
	}

	if (m_pszDomainName != NULL) {
		delete [] m_pszDomainName;
	}

	if (m_pszHostName != NULL) {
		delete [] m_pszHostName;
	}

	if (m_pszLastModified != NULL) {
		delete [] m_pszLastModified;
	}

	if (m_pszHeaders != NULL) {
		delete [] m_pszHeaders;
	}

	if (m_pszLogin != NULL) {
		delete [] m_pszLogin;
	}

	if (m_pszPassword != NULL) {
		delete [] m_pszPassword;
	}

	if (m_ppszFormName != NULL) {
	
		for (i = 0; i < m_iNumForms; i ++) {
			if (m_ppszFormName[i] != NULL) {
				delete [] m_ppszFormName[i];
			}
		}
		delete [] m_ppszFormName;
	}

	if (m_ppszFormValue != NULL) {
		
		for (i = 0; i < m_iNumForms; i ++) {
			if (m_ppszFormValue[i] != NULL) {
				delete [] m_ppszFormValue[i];
			}
		}
		delete [] m_ppszFormValue;
	}

	if (m_piFormType != NULL) {
		delete [] m_piFormType;
	}

	if (m_ppszFormFileName != NULL) {
		
		for (i = 0; i < m_iNumForms; i ++) {
			if (m_ppszFormFileName[i] != NULL) {
				delete [] m_ppszFormFileName[i];
			}
		}
		delete [] m_ppszFormFileName;
	}

	if (m_ppszCookieName != NULL) {
	
		for (i = 0; i < m_iNumCookies; i ++) {
			if (m_ppszCookieName[i] != NULL) {
				delete [] m_ppszCookieName[i];
			}
		}
		delete [] m_ppszCookieName;
	}

	if (m_ppszCookieValue != NULL) {
		
		for (i = 0; i < m_iNumCookies; i ++) {
			if (m_ppszCookieValue[i] != NULL) {
				delete [] m_ppszCookieValue[i];
			}
		}
		delete [] m_ppszCookieValue;
	}
}

int HttpRequest::GetMethod() {
	return m_iMethod;
}

int HttpRequest::GetVersion() {
	return m_iVersion;
}

char* HttpRequest::GetUri() {
	return m_pszUri;
}

char* HttpRequest::GetFileName() {
	return m_pszFileName;
}

char* HttpRequest::GetBrowserName() {
	return m_pszBrowserName;
}

char* HttpRequest::GetClientIP() {
	return m_pszIPAddress;
}

char* HttpRequest::GetClientDomainName() {
	return m_pszDomainName;
}

bool HttpRequest::GetCached() {
	return m_bCached;
}

char* HttpRequest::GetLastModified() {
	return m_pszLastModified;
}

char* HttpRequest::GetHeaders() {
	return m_pszHeaders;
}

void* HttpRequest::GetPageSource() {
	return m_pPageSource;
}

char* HttpRequest::GetLogin() {
	return m_pszLogin;
}

char* HttpRequest::GetPassword() {
	return m_pszPassword;
}

void HttpRequest::SetStatus (int iStatus) {
	m_iStatus = iStatus;
}

int HttpRequest::GetStatus() {
	return m_iStatus;
}


int HttpRequest::ParseHeaders (ServerSocket* pssSocket, void* pvDefaultPageSource, void* pvPageSourceTable) {

	PageSource* pDefaultPageSource = (PageSource*) pvDefaultPageSource;
	HashTable<PageSource>* pPageSourceTable = (HashTable<PageSource>*) pvPageSourceTable;

	char pszMethod [MAX_METHOD_LENGTH];
	char pszUri [MAX_FILENAME_LENGTH];
	char pszVer [MAX_VERSION_LENGTH];

	char* pszLine = NULL;
	char* pszDecode = NULL;

	// Get the client's IP Address and domain name
	char* pszTemp = pssSocket->GetClientIP();
	m_pszIPAddress = new char [strlen (pszTemp) + 1];
	sprintf (m_pszIPAddress, pszTemp);

	pszTemp = pssSocket->GetClientDomainName();
	m_pszDomainName = new char [strlen (pszTemp) + 1];
	sprintf (m_pszDomainName, pszTemp);

	// Tell the socket to receive the request
	pssSocket->RecvRequest();

	// Fetch the first line of the request
	pszLine = pssSocket->GetNextLine();

	// Check for null request
	if (pszLine == NULL) {
		return ERR;
	}

	// Parse the first line
	sscanf (pszLine, "%s %s %s", pszMethod, pszUri, pszVer);

	// Handle method
	if (stricmp (pszMethod, "GET") == 0) {
		m_iMethod = GET;
	}
	else if (stricmp (pszMethod, "POST") == 0) {
		m_iMethod = POST;
	}
	else if (stricmp (pszMethod, "PUT") == 0) {
		m_iMethod = PUT;
	}
	else if (stricmp (pszMethod, "HEAD") == 0) {
		m_iMethod = HEAD;
	}
	else if (stricmp (pszMethod, "TRACE") == 0) {
		m_iMethod = TRACE;

		// Save headers
		char* pszHeaders = pssSocket->GetData();
		m_pszHeaders = new char [strlen (pszHeaders) + 1];
		sprintf (m_pszHeaders, pszHeaders);
	}
	else {
		return UNSUPPORTED_HTTP_METHOD;
	}

	// Handle Uri
	m_pszUri = Utility::WWWEscapeCharToAscii (pszUri);
	sprintf (pszUri, m_pszUri);

	// The first word of the URI is the name of the PageSource
	char* pszPageSource = strtok (pszUri, "/");
	if (pszPageSource == NULL) {
		m_pPageSource = pDefaultPageSource;
	} else {
		m_pPageSource = pPageSourceTable->GetElement (pszPageSource);
		if (m_pPageSource == NULL) {
			m_pPageSource = pDefaultPageSource;
		}
	}

	// Handle filename
	char* pszBasePath = ((PageSource*) m_pPageSource)->GetBasePath();
	
	pszTemp = strtok (NULL, "");
	if (pszTemp == NULL) {
		m_pszFileName = new char [strlen (m_pszUri) + strlen (pszBasePath) + 3];
	} else {
		m_pszFileName = new char [strlen (m_pszUri) + strlen (pszBasePath) + strlen (pszTemp) + 3];
	}

	sprintf (m_pszFileName, pszBasePath);
	
	if (m_pPageSource == pDefaultPageSource) {
		char* pszFixedUri = Utility::SlashToWhack (m_pszUri);
		strcat (m_pszFileName, pszFixedUri);
		delete [] pszFixedUri;
	} else {
		if (m_pszUri[strlen (m_pszUri) - 1] == '/' || pszTemp != NULL) {
			strcat (m_pszFileName, "\\");

			if (pszTemp != NULL) {
				pszTemp = Utility::SlashToWhack (pszTemp);
				strcat (m_pszFileName, pszTemp);
				delete [] pszTemp;
			}
		}
	}

	// Handle version
	if (stricmp (pszVer, "HTTP/1.1") == 0) {
		m_iVersion = HTTP11;
	}
	else if (stricmp (pszVer, "HTTP/1.0") == 0) {
		m_iVersion = HTTP10;
	}
	else if (stricmp (pszVer, "HTTP/0.9") == 0) {
		m_iVersion = HTTP09;
	}
	else {
		return UNSUPPORTED_HTTP_VERSION;
	}

	// Parse the remaining headers
	bool bFormsParsed = false;

	char* pszHeader = NULL;
	char* pszValue = NULL;
	char* pszSeparator = NULL;

	int iContentLength = 0;

	char pszBuf [MAX_REQUEST_LENGTH];

	while (true) {

		// Get a line
		pszLine = pssSocket->GetNextLine();

		// Check for end of headers
		if (pszLine == NULL) {
			if (!bFormsParsed && m_iMethod == POST) {
				pssSocket->RecvRequest();
				pszLine = pssSocket->GetNextLine();

				if (pszLine == NULL) {
					break;
				}
			} else {
				break;
			}
		}

		// Check for empty line (form submission)
		if (m_iMethod == POST && pszLine[0] == '\0') {
			
			if (pszSeparator == NULL) {
				bFormsParsed = this->HandleSimpleForms (pssSocket);
			} else {
				bFormsParsed = this->HandleMultipartForms (pssSocket, pszSeparator, iContentLength);
			}

		} else {
		
			//////////////////
			// Normal header //
			//////////////////
			
			// Separate header name from value
			sprintf (pszBuf, pszLine);
			pszHeader = strtok (pszBuf, " ");
			pszValue = strtok (NULL, "");
			
			// Handle headers
			switch (pszLine[0]) {
		
			// Authorization
			case 'A':
			case 'a':
				if (stricmp (pszHeader, "Authorization:") == 0) {
					
					pszTemp = new char [strlen (pszValue) + 1];
					sscanf (pszValue, "Basic %s", pszTemp);

					Utility::DecodeBase64 (pszTemp, &pszDecode);
					delete [] pszTemp;

					pszTemp = strtok (pszDecode, ":");
					if (pszTemp == NULL) {
						m_pszLogin = new char [1];
						m_pszLogin[0] = '\0';
					} else {
						m_pszLogin = new char [strlen (pszTemp) + 1];
						sprintf (m_pszLogin, pszTemp);
					}

					pszTemp = strtok (NULL, ":");
					
					if (pszTemp == NULL) {
						m_pszPassword = new char [1];
						m_pszPassword[0] = '\0';
					} else {
						m_pszPassword = new char [strlen (pszTemp) + 1];
						sprintf (m_pszPassword, pszTemp);
					}

					delete [] pszDecode;
				}

				break;

			// Connection, Content-Length, Cookie
			case 'C':
			case 'c':
				
				if (stricmp (pszHeader, "Connection:") == 0) {
					m_bKeepAlive = (stricmp (pszValue, "Keep-Alive") == 0);
				}
				else if (stricmp (pszHeader, "Content-Type:") == 0) {
					
					// Check for multipart
					if (strstr (pszValue, "multipart/form-data") != NULL) {
						
						// Get boundary
						pszSeparator = new char [strlen (pszValue) + 1];
						sscanf (pszValue, "multipart/form-data; boundary=%s", pszSeparator);
					}
				}
				else if (stricmp (pszHeader, "Content-Length:") == 0) {
					iContentLength = atoi (pszValue);
				}
				else if (stricmp (pszHeader, "Cookie:") == 0) {
					
					// Count the number of semicolons
					int i, iNumSemicolons = 0, iLength = strlen (pszValue);
					for (i = 0; i < iLength; i ++) {
						if (pszValue[i] == ';') {
							iNumSemicolons ++;
						}
					}

					// Allocate space for cookie primitives
					char** ppszCookie = new char* [iNumSemicolons + 1];

					// Tokenize by semicolons
					int iNumCookies = 0;
					pszTemp = strtok (pszValue, ";");
					while (pszTemp != NULL) {
						ppszCookie[iNumCookies] = new char [strlen (pszTemp) + 1];
						sprintf (ppszCookie[iNumCookies], pszTemp);
						iNumCookies ++;
						pszTemp = strtok (NULL, ";");
					}

					// Tokenize by equals
					m_iNumCookies = iNumCookies;
					m_ppszCookieName = new char* [m_iNumCookies];
					m_ppszCookieValue = new char* [m_iNumCookies];

					for (i = 0; i < m_iNumCookies; i ++) {
						pszTemp = strtok (ppszCookie[i], "=");
						if (i > 0) {
							// Adjust for space after semicolon
							pszTemp += sizeof (char);
						}
						m_ppszCookieName[i] = new char [strlen (pszTemp) + 1];
						sprintf (m_ppszCookieName[i], pszTemp);

						pszTemp = strtok (NULL, "");
						m_ppszCookieValue[i] = new char [strlen (pszTemp) + 1];
						sprintf (m_ppszCookieValue[i], pszTemp);
					}					
				}
				
				break;
				
			// Host
			case 'H':
			case 'h':
				
				if (stricmp (pszHeader, "Host:") == 0) {
					m_pszHostName = new char [strlen (pszValue) + 1];
					sprintf (m_pszHostName, pszValue);
				}
				
				break;
				
			case 'I':
			case 'i':
				if (stricmp (pszHeader, "If-Modified-Since") == 0) {
					m_bCached = !Utility::ModifiedSince (m_pszFileName, pszValue, &m_pszLastModified);

				}

				break;

			// User-Agent
			case 'U':
			case 'u':
				
				if (stricmp (pszHeader, "User-Agent:") == 0) {
					m_pszBrowserName = new char [strlen (pszValue) + 1];
					sprintf (m_pszBrowserName, pszValue);
				}
				break;
				
			default:
				break;
			}
		}
	}

	if (pszSeparator != NULL) {
		delete [] pszSeparator;
	}

	return OK;
}


/////////////
// Cookies //
/////////////

int HttpRequest::GetNumCookies() {
	return m_iNumCookies;
}

char** HttpRequest::GetCookieNames() {
	return m_ppszCookieName;
}

char** HttpRequest::GetCookieValues() {
	return m_ppszCookieValue;
}

bool HttpRequest::GetKeepAlive() {
	return m_bKeepAlive;
}


///////////
// Forms //
///////////

int HttpRequest::GetNumForms() {
	return m_iNumForms;
}

int* HttpRequest::GetFormTypes() {
	return m_piFormType;
}

char** HttpRequest::GetFormNames() {
	return m_ppszFormName;
}

char** HttpRequest::GetFormValues() {
	return m_ppszFormValue;
}

char** HttpRequest::GetFormFileNames() {
	return m_ppszFormFileName;
}

bool HttpRequest::HandleSimpleForms (ServerSocket* pssSocket) {

	int i, iTemp, iNumForms = 2;
	char** ppszTemp = new char* [iNumForms];
	char** ppszTemp2;
	
	// Get next line
	char* pszLine = pssSocket->GetNextLine();
	
	if (pszLine == NULL) {
		delete [] ppszTemp;
		ppszTemp = NULL;
		return false;
	}

	// Tokenize ampersands
	ppszTemp[0] = strtok (pszLine, "&");
	
	// Separate the forms
	if (ppszTemp[0] != NULL) {
		
		m_iNumForms = 1;
		
		while (true) {
			
			ppszTemp[m_iNumForms] = strtok (NULL, "&");
			
			if (ppszTemp[m_iNumForms] == NULL) {
				break;
			}
			
			m_iNumForms ++;
			
			// Resize?
			if (m_iNumForms == iNumForms) {
				
				iTemp = iNumForms * 2;
				ppszTemp2 = new char* [iTemp];
				
				memcpy (ppszTemp2, ppszTemp, iNumForms * sizeof (char*));
				
				iNumForms = iTemp;
				
				delete [] ppszTemp;
				ppszTemp = ppszTemp2;
			}
		}
	}
	
	if (m_iNumForms > 0) {
		
		// Allocate storage for the names and values
		if (m_ppszFormName != NULL) {
			for (i = 0; i < m_iNumForms; i ++) {
				delete [] m_ppszFormName[i];
			}
			delete [] m_ppszFormName;
		}
		
		if (m_ppszFormValue != NULL) {		
			for (i = 0; i < m_iNumForms; i ++) {
				delete [] m_ppszFormValue[i];
			}
			delete [] m_ppszFormValue;
		}
		
		m_ppszFormName = new char* [m_iNumForms];
		m_ppszFormValue = new char* [m_iNumForms];
		m_piFormType = new int [m_iNumForms];
		m_ppszFormFileName = new char* [m_iNumForms];

		// Separate the names and values
		for (i = 0; i < m_iNumForms; i ++) {
			m_ppszFormName[i] = Utility::WWWEscapeCharToAscii (strtok (ppszTemp[i], "="));
			m_ppszFormValue[i] = Utility::WWWEscapeCharToAscii(strtok (NULL, "="));
			m_piFormType[i] = SIMPLE_FORM;
			m_ppszFormFileName[i] = NULL;
		}
	}
	
	delete [] ppszTemp;		// No need to delete subarrays

	return true;
}


bool HttpRequest::HandleMultipartForms (ServerSocket* pssSocket, char* pszSeparator, int iContentLength) {

	int iTemp, iPartialBytes, iNumBytes, iTotalBytes, iSpace = 4;
	char** ppszTempName;
	char** ppszTempValue;
	int* piTempType;
	char** ppszTempFileName;
	
	char* pszTemp;
	char* pszTemp2;
	TempFile* pTempFile;

	bool bSeparator;

	// Get separator line
	char* pszLine = pssSocket->GetNextLine();
	if (pszLine == NULL) {
		pssSocket->RecvRequest();
		pszLine = pssSocket->GetNextLine();

		if (pszLine == NULL) {
			return false;
		}
	} else {
		if (strstr (pszLine, pszSeparator) == NULL) {
			return false;
		}
	}

	// Allocate initial space for forms
	m_ppszFormName = new char* [iSpace];
	m_ppszFormValue = new char* [iSpace];
	m_piFormType = new int [iSpace];
	m_ppszFormFileName = new char* [iSpace];

	m_iNumForms = 0;
	iTotalBytes = 0;

	while (true) {
		
		iNumBytes = 1;
		iPartialBytes = 0;
		bSeparator = false;

		// Get value disposition
		pszLine = pssSocket->GetNextLine();

		// If null
		if (pszLine == NULL) {

			if (iTotalBytes > iContentLength) {
				return (m_iNumForms > 0);
			} else {
				pssSocket->RecvRequest();
				pszLine = pssSocket->GetNextLine();

				if (pszLine == NULL) {
					return false;
				}
			}
		}
		
		// Resize storage?
		if (m_iNumForms == iSpace) {
			
			// Allocate new storage
			iTemp = iSpace * 2;
			ppszTempName = new char* [iTemp];
			ppszTempValue = new char* [iTemp];
			piTempType = new int [iTemp];
			ppszTempFileName = new char* [iTemp];

			// Copy data over
			memcpy (ppszTempName, m_ppszFormName, iSpace * sizeof (char*));
			memcpy (ppszTempValue, m_ppszFormValue, iSpace * sizeof (char*));
			memcpy (piTempType, m_piFormType, iSpace * sizeof (int));
			memcpy (ppszTempFileName, m_ppszFormFileName, iSpace * sizeof (char*));
			
			// Delete old storage
			delete [] m_ppszFormName;
			delete [] m_ppszFormValue;
			delete [] m_piFormType;
			delete [] m_ppszFormFileName;

			// Reassign pointers
			m_ppszFormName = ppszTempName;
			m_ppszFormValue = ppszTempValue;
			m_piFormType = piTempType;
			m_ppszFormFileName = ppszTempFileName;

			// Save new size
			iSpace = iTemp;
		}

		// Get form name
		pszTemp = strtok (pszLine, ";");
		pszTemp = strtok (NULL, "; ");
		pszTemp2 = new char [strlen (pszTemp) + 1];
		sscanf (pszTemp, "name=\"%s", pszTemp2);
		m_ppszFormName[m_iNumForms] = Utility::WWWEscapeCharToAscii (pszTemp2);
		m_ppszFormName[m_iNumForms][strlen (m_ppszFormName[m_iNumForms]) - 1] = '\0';
		delete [] pszTemp2;

		// Get filename, if available
		pszTemp = strtok (NULL, "");
		if (pszTemp != NULL) {
			pszTemp2 = new char [strlen (pszTemp) + 1];
			sscanf (pszTemp, " filename=\"%s", pszTemp2);
			m_ppszFormFileName[m_iNumForms] = Utility::WWWEscapeCharToAscii (pszTemp2);
			m_ppszFormFileName[m_iNumForms][strlen (m_ppszFormFileName[m_iNumForms]) - 1] = '\0';
			delete [] pszTemp2;
			
			m_piFormType[m_iNumForms] = FILE_FORM;
			
			// Create new temporary file
			pTempFile = new TempFile();
			
			// Save filename
			pszTemp = pTempFile->GetName();
			m_ppszFormValue[m_iNumForms] = new char [strlen (pszTemp) + 1];
			sprintf (m_ppszFormValue [m_iNumForms], pszTemp);
			
			// Write file contents to temp file
			pTempFile->Open();
			
			pszLine = pssSocket->GetNextLine();		// Content type
			if (pszLine == NULL) {
				pssSocket->RecvRequest();
				pszLine = pssSocket->GetNextLine();

				if (pszLine == NULL) {
					return false;
				}
			}
			
			pszLine = pssSocket->GetNextLine();		// White space
			if (pszLine == NULL) {
				pssSocket->RecvRequest();
				pszLine = pssSocket->GetNextLine();

				if (pszLine == NULL) {
					return false;
				}
			}

			// Loop until end of current file
			while (iNumBytes > 0) {
				
				iNumBytes = pssSocket->GetNumRemainingBytes();
				pszLine = pssSocket->GetRemainingData();	// Content
			
				if (pszLine == NULL) {
					pssSocket->RecvRequest();
					iNumBytes = pssSocket->GetNumRemainingBytes();
					pszLine = pssSocket->GetRemainingData();

					if (pszLine == NULL) {
						return false;
					}
				}

				iPartialBytes += pssSocket->GetNumBytes();
				iTotalBytes += pssSocket->GetNumBytes();

				if (iNumBytes > 0) {
					
					// Loop until end marker is found or definitely not present
					pszTemp = pszLine;
					while (true) {
							
						if ((pszTemp = (char*) memchr (pszTemp, pszSeparator[0], iNumBytes - (pszTemp - pszLine)))
							== NULL) {
							
							// No end marker
							break;
						
						} else {
						
							// TODO:  Somehow avoid breaks in text strings when recv'ing
							if (memcmp (pszTemp, pszSeparator, strlen (pszSeparator) * sizeof (char)) == 0) {
								
								bSeparator = true;

								// Found end marker
								iNumBytes = pszTemp - pszLine - 4;
								pssSocket->SetPosition ((int) pszLine + iNumBytes + 4);

								pszTemp = pssSocket->GetNextLine();
								if (pszTemp == NULL) {
									pssSocket->RecvRequest();
									pszLine = pssSocket->GetNextLine();
									if (pszLine == NULL) {
										return false;
									}
								}

								break;
							}
						}

						if (pszTemp - pszLine == iNumBytes) {
							// End of buffer
							break;
						} else {
							// Continue advancing
							pszTemp ++;
						}
					}

					// Write data to disk
					pTempFile->Write (pszLine, iNumBytes);

					if (bSeparator) {
						// Finished with the file
						break;
					} else {
						// More data waiting for us
						pssSocket->RecvRequest();
					}
				}
			}
		
			// Close temp file
			pTempFile->Close();
			delete pTempFile;
		
		} else {
			
			m_ppszFormFileName[m_iNumForms] = NULL;
			m_piFormType[m_iNumForms] = SIMPLE_FORM;

			// Move forward to form's value, recv'ing if we encounter an unexpected eof
			pszLine = pssSocket->GetNextLine();		// White space
			if (pszLine == NULL) {
				pssSocket->RecvRequest();
				pszLine = pssSocket->GetNextLine();

				if (pszLine == NULL) {
					return false;
				}
			}

			pszLine = pssSocket->GetNextLine();		// Value
			if (pszLine == NULL) {
				pssSocket->RecvRequest();
				pszLine = pssSocket->GetNextLine();

				if (pszLine == NULL) {
					return false;
				}
			}
			m_ppszFormValue [m_iNumForms] = new char [strlen (pszLine) + 1];
			sprintf (m_ppszFormValue [m_iNumForms], pszLine);

			// Get separator line
			pszLine = pssSocket->GetNextLine();
			if (pszLine == NULL) {
				pssSocket->RecvRequest();
				pszLine = pssSocket->GetNextLine();

				if (pszLine == NULL) {
					return false;
				}
			}
		}

		m_iNumForms ++;
	}

	return true;
}