// HttpRequest.h: interface for the HttpRequest class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_HTTPREQUEST_H__7E3E4056_A569_11D1_9C4E_0060083E8062__INCLUDED_)
#define AFX_HTTPREQUEST_H__7E3E4056_A569_11D1_9C4E_0060083E8062__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "ServerSocket.h"
#include "TempFile.h"

class HttpRequest {

	friend class HttpServer;

private:

	// Method, version, Uri
	int m_iMethod;
	int m_iVersion;
	char* m_pszUri;

	// Uri as hard drive path
	char* m_pszFileName;

	// Browsername
	char* m_pszBrowserName;

	// Client IP address, domain name
	char* m_pszIPAddress;
	char* m_pszDomainName;

	// Name of host requested by client
	char* m_pszHostName;

	// True if file requested is cached (and last modification date of file is non-null also)
	bool m_bCached;
	char* m_pszLastModified;

	// Headers of request, if request method is TRACE
	char* m_pszHeaders;

	// Authorization
	char* m_pszLogin;
	char* m_pszPassword;

	// Forms
	int m_iNumForms;
	char** m_ppszFormName;
	char** m_ppszFormValue;
	char** m_ppszFormFileName;
	int* m_piFormType;
	
	// Cookies
	int m_iNumCookies;
	char** m_ppszCookieName;
	char** m_ppszCookieValue;

	// Keep-alive?
	bool m_bKeepAlive;
	
	// PageSource
	void* m_pPageSource;

	// Status
	int m_iStatus;

	virtual bool HandleSimpleForms (ServerSocket* pssSocket);
	virtual bool HandleMultipartForms (ServerSocket* pssSocket, char* pszSeparator, int iContentLength);

	// HttpServer interface
	virtual inline bool GetCached();
	virtual inline char* GetLastModified();
	virtual inline char* GetHeaders();
	
	virtual inline bool GetKeepAlive();
	virtual inline void* GetPageSource();

	virtual inline void SetStatus (int iStatus);
	virtual inline int GetStatus();

	virtual int ParseHeaders (ServerSocket* pssSocket, void* pvDefaultPageSource, void* pvPageSourceTable);

public:

	HttpRequest();
	virtual ~HttpRequest();

	// Public interface
	virtual inline int GetMethod();
	virtual inline int GetVersion();
	virtual inline char* GetUri();
	virtual inline char* GetFileName();
	virtual inline char* GetBrowserName();
	virtual inline char* GetClientIP();
	virtual inline char* GetClientDomainName();

	virtual inline int GetNumForms();
	virtual inline int* GetFormTypes();
	virtual inline char** GetFormNames();
	virtual inline char** GetFormValues();
	virtual inline char** GetFormFileNames();
	
	virtual inline int GetNumCookies();
	virtual inline char** GetCookieNames();
	virtual inline char** GetCookieValues();

	virtual inline char* GetLogin();
	virtual inline char* GetPassword();
};

#endif // !defined(AFX_HTTPREQUEST_H__7E3E4056_A569_11D1_9C4E_0060083E8062__INCLUDED_)
