// HttpResponse.cpp: implementation of the HttpResponse class.
//
//////////////////////////////////////////////////////////////////////

#include "HttpResponse.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HttpResponse::HttpResponse() {
	
	m_bOverride = false;
	m_iType = 0;
	m_iStatus = 0;
	m_iLength = 0;

	m_pszResponseData = NULL;
	m_pszFileName = NULL;
	m_pszMimeType = NULL;
	m_pszRedirectUri = NULL;

	m_iNumCookiesSet = 0;
	m_iNumCookiesDel = 0;
	m_iSetCookieSpace = 0;
	m_iDelCookieSpace = 0;

	m_ppszCookieSetName = NULL;
	m_ppszCookieSetValue = NULL;
	m_ppszCookieSetPath = NULL;
	m_piCookieTTL = NULL;

	m_ppszCookieDelName = NULL;
	m_ppszCookieDelPath = NULL;
}

HttpResponse::~HttpResponse() {

	if (m_pszResponseData != NULL) {
		delete [] m_pszResponseData;
	}

	if (m_pszFileName != NULL) {
		delete [] m_pszFileName;
	}

	if (m_pszMimeType != NULL) {
		delete [] m_pszMimeType;
	}

	if (m_pszRedirectUri != NULL) {
		delete [] m_pszRedirectUri;
	}

	int i;
	for (i = 0; i < m_iNumCookiesSet; i ++) {
		delete [] m_ppszCookieSetName[i];
		delete [] m_ppszCookieSetValue[i];
		delete [] m_ppszCookieSetPath[i];
	}

	if (m_ppszCookieSetName != NULL) {
		delete [] m_ppszCookieSetName;
	}
	
	if (m_ppszCookieSetValue != NULL) {
		delete [] m_ppszCookieSetValue;
	}

	if (m_ppszCookieSetPath != NULL) {
		delete [] m_ppszCookieSetPath;
	}

	if (m_piCookieTTL != NULL) {
		delete [] m_piCookieTTL;
	}

	for (i = 0; i < m_iNumCookiesDel; i ++) {
		delete [] m_ppszCookieDelName[i];
		delete [] m_ppszCookieDelPath[i];
	}

	if (m_ppszCookieDelName != NULL) {
		delete [] m_ppszCookieDelName;
	}

	if (m_ppszCookieDelPath != NULL) {
		delete [] m_ppszCookieDelPath;
	}
}


void HttpResponse::SetOverride (bool bOverride) {
	m_bOverride = bOverride;
}


void HttpResponse::SetType (int iType) {
	m_iType = iType;
}

void HttpResponse::SetStatus (int iStatus) {
	m_iStatus = iStatus;
}

void HttpResponse::SetData (char* pszData, int iLength) {
	if (pszData != NULL && iLength > 0) {
		m_iLength = iLength;
		m_pszResponseData = new char [m_iLength + 1];
		memcpy (m_pszResponseData, pszData, m_iLength * sizeof (char));
	}
}

void HttpResponse::SetData (char* pszData) {

	if (pszData != NULL) {
		m_iLength = strlen (pszData);
		m_pszResponseData = new char [m_iLength + 1];
		strcpy (m_pszResponseData, pszData);
	}
}

void HttpResponse::ClearData() {
	if (m_pszResponseData != NULL) {
		delete [] m_pszResponseData;
		m_pszResponseData = NULL;
		m_iLength = 0;
	}
}

void HttpResponse::WriteData (char* pszData) {
	if (pszData != NULL) {
		int iLength = strlen (pszData);
		if (iLength > 0) {
			char* pszTemp = new char [m_iLength + iLength + 1];
			if (m_pszResponseData != NULL) {
				strcpy (pszTemp, m_pszResponseData);
				strcat (pszTemp, pszData);
				delete [] m_pszResponseData;
			} else {
				strcpy (pszTemp, pszData);
			}

			m_pszResponseData = pszTemp;
			m_iLength += iLength;
		}
	}
}

void HttpResponse::WriteData (char* pszData, int iLength) {
	if (pszData != NULL && iLength > 0) {
		char* pszTemp = new char [m_iLength + iLength + 1];
		if (m_pszResponseData != NULL) {
			memcpy (pszTemp, m_pszResponseData, m_iLength * sizeof (char));
			memcpy (&(pszTemp[m_iLength]), pszData, iLength * sizeof (char));
			delete [] m_pszResponseData;
		} else {
			memcpy (pszTemp, pszData, iLength * sizeof (char));
		}

		m_pszResponseData = pszTemp;
		m_iLength += iLength;
	}
}

void HttpResponse::SetFileName (char* pszFileName) {
	if (pszFileName != NULL) {
		m_pszFileName = new char [strlen (pszFileName) + 1];
		sprintf (m_pszFileName, pszFileName);
	}
}

void HttpResponse::SetMimeType (char* pszMimeType) {
	if (pszMimeType != NULL) {
		m_pszMimeType = new char [strlen (pszMimeType) + 1];
		sprintf (m_pszMimeType, pszMimeType);
	}
}

void HttpResponse::SetRedirectUri (char* pszRedirectUri) {
	if (pszRedirectUri != NULL) {
		m_pszRedirectUri = new char [strlen (pszRedirectUri) + 1];
		sprintf (m_pszRedirectUri, pszRedirectUri);
	}
}

bool HttpResponse::Override() {
	return m_bOverride;
}

int HttpResponse::GetType() {
	return m_iType;
}

int HttpResponse::GetStatus() {
	return m_iStatus;
}

char* HttpResponse::GetData() {
	return m_pszResponseData;
}

int HttpResponse::GetLength() {
	return m_iLength;
}

char* HttpResponse::GetFileName() {
	return m_pszFileName;
}

char* HttpResponse::GetMimeType() {
	return m_pszMimeType;
}

char* HttpResponse::GetRedirectUri() {
	return m_pszRedirectUri;
}

int HttpResponse::SetCookie (char* pszCookieName, char* pszCookieValue, int iTTLinSeconds, char* pszCookiePath) {

	if (pszCookieName == NULL || pszCookieValue == NULL) {
		return ERR;
	}

	if (m_iNumCookiesSet == m_iSetCookieSpace) {

		if (m_iSetCookieSpace == 0) {

			m_iSetCookieSpace = 4;
			m_ppszCookieSetName = new char* [m_iSetCookieSpace];
			m_ppszCookieSetValue = new char* [m_iSetCookieSpace];
			m_ppszCookieSetPath = new char* [m_iSetCookieSpace];
			m_piCookieTTL  = new int [m_iSetCookieSpace];

		} else {

			int iNewLength = m_iSetCookieSpace * 2;
			char** ppszCookieSetName = new char* [iNewLength];
			char** ppszCookieSetValue = new char* [iNewLength];
			char** ppszCookieSetPath = new char* [iNewLength];
			int* piCookieTTL  = new int [iNewLength];

			memcpy (ppszCookieSetName, m_ppszCookieSetName, m_iSetCookieSpace * sizeof (char*));
			memcpy (ppszCookieSetValue, m_ppszCookieSetValue, m_iSetCookieSpace * sizeof (char*));
			memcpy (ppszCookieSetPath, m_ppszCookieSetPath, m_iSetCookieSpace * sizeof (char*));
			memcpy (piCookieTTL, m_piCookieTTL, m_iSetCookieSpace * sizeof (int));

			delete [] m_ppszCookieSetName;
			delete [] m_ppszCookieSetValue;
			delete [] m_ppszCookieSetPath;
			delete [] m_piCookieTTL;

			m_ppszCookieSetName = ppszCookieSetName;
			m_ppszCookieSetValue = ppszCookieSetValue;
			m_ppszCookieSetPath = ppszCookieSetPath;
			m_piCookieTTL = piCookieTTL;

			m_iSetCookieSpace = iNewLength;
		}
	}

	m_ppszCookieSetName[m_iNumCookiesSet] = new char [strlen (pszCookieName) + 1];
	sprintf (m_ppszCookieSetName[m_iNumCookiesSet], pszCookieName);

	m_ppszCookieSetValue[m_iNumCookiesSet] = new char [strlen (pszCookieValue) + 1];
	sprintf (m_ppszCookieSetValue[m_iNumCookiesSet], pszCookieValue);

	m_piCookieTTL[m_iNumCookiesSet] = iTTLinSeconds;

	if (pszCookiePath == NULL) {
		m_ppszCookieSetPath[m_iNumCookiesSet] = NULL;
	} else {
		m_ppszCookieSetPath[m_iNumCookiesSet] = new char [strlen (pszCookiePath) + 1];
		sprintf (m_ppszCookieSetPath[m_iNumCookiesSet], pszCookiePath);
	}

	m_iNumCookiesSet ++;

	return OK;
}

int HttpResponse::DeleteCookie (char* pszCookieName, char* pszCookiePath) {

	if (pszCookieName == NULL) {
		return ERR;
	}

	if (m_iNumCookiesDel == m_iDelCookieSpace) {

		if (m_iDelCookieSpace == 0) {

			m_iDelCookieSpace = 4;
			m_ppszCookieDelName = new char* [m_iDelCookieSpace];
			m_ppszCookieDelPath = new char* [m_iDelCookieSpace];

		} else {

			int iNewLength = m_iDelCookieSpace * 2;
			char** ppszCookieDelName = new char* [iNewLength];
			char** ppszCookieDelPath = new char* [iNewLength];

			memcpy (ppszCookieDelName, m_ppszCookieDelName, m_iDelCookieSpace * sizeof (char*));
			memcpy (ppszCookieDelPath, m_ppszCookieDelPath, m_iDelCookieSpace * sizeof (char*));

			delete [] m_ppszCookieDelName;
			delete [] m_ppszCookieDelPath;

			m_ppszCookieDelName = ppszCookieDelName;
			m_ppszCookieDelPath = ppszCookieDelPath;

			m_iDelCookieSpace = iNewLength;
		}
	}

	m_ppszCookieDelName[m_iNumCookiesDel] = new char [strlen (pszCookieName) + 1];
	sprintf (m_ppszCookieDelName[m_iNumCookiesDel], pszCookieName);

	m_ppszCookieDelPath[m_iNumCookiesDel] = new char [strlen (pszCookiePath) + 1];
	sprintf (m_ppszCookieDelPath[m_iNumCookiesDel], pszCookiePath);

	m_iNumCookiesDel ++;

	return OK;
}


int HttpResponse::GetNumCookiesSet() {
	return m_iNumCookiesSet;
}

int HttpResponse::GetNumCookiesDeleted() {
	return m_iNumCookiesDel;
}

void HttpResponse::GetCookiesSet (char*** pppszCookieName, char*** pppszCookieValue, int** ppiTTLinSeconds, 
								  char*** pppszCookiePath) {

	*pppszCookieName = m_ppszCookieSetName;
	*pppszCookieValue = m_ppszCookieSetValue;
	*ppiTTLinSeconds = m_piCookieTTL;
	*pppszCookiePath = m_ppszCookieSetPath;
}

void HttpResponse::GetCookiesDeleted (char*** pppszCookieName, char*** pppszCookiePath) {
	*pppszCookieName = m_ppszCookieDelName;
	*pppszCookiePath = m_ppszCookieDelPath;
}