// HttpServer.cpp: implementation of the HttpServer class.
//
//////////////////////////////////////////////////////////////////////

#include "HttpServer.h"
#include "ConfigFile.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

HttpServer::HttpServer() {

	m_pssSocket = NULL;
	
	m_bExit = false;
	m_bServing = false;
	m_pExitMutex = new Mutex();
	m_pThreadsMutex = new Mutex();
	m_iNumThreads = 0;

	m_psz401Text = new char [55];
	strcpy (m_psz401Text, "<strong>Error 401:</strong><p>Unauthorized.");

	m_psz403Text = new char [55];
	strcpy (m_psz403Text, "<strong>Error 403:</strong><p>Access Forbidden.");

	m_psz404Text = new char [55];
	strcpy (m_psz404Text, "<strong>Error 404:</strong><p>Not Found.");

	m_psz500Text = new char [60];
	strcpy (m_psz500Text, "<strong>Error 500:</strong><p>Internal Server Error.");

	m_psz501Text = new char [55];
	strcpy (m_psz501Text, "<strong>Error 501:</strong><p>Not Implemented.");

	m_psz505Text = new char [60];
	strcpy (m_psz505Text, "<strong>Error 505:</strong><p>HTTP Version Not Supported.");

	m_pszServerName = new char [20];
	strcpy (m_pszServerName, "Alajar 1.0");

	m_pDefaultPageSource = NULL;
	m_pPageSourceTable = NULL;

	// SSI
	m_pszSSIBegin = new char [5];
	strcpy (m_pszSSIBegin, "<!--");
	m_iSSIBegin = strlen (m_pszSSIBegin);

	m_pszSSIEnd = new char [4];
	strcpy (m_pszSSIEnd, "-->");
	m_iSSIEnd = strlen (m_pszSSIEnd);

	int iEndLen = strlen (m_pszSSIEnd);

	m_pszSSITime = new char [5 + iEndLen];
	strcpy (m_pszSSITime, "Time");
	strcat (m_pszSSITime, m_pszSSIEnd);
	m_iSSITime = strlen (m_pszSSITime);

	m_pszSSIDate = new char [5 + iEndLen];
	strcpy (m_pszSSIDate, "Date");
	strcat (m_pszSSIDate, m_pszSSIEnd);
	m_iSSIDate = strlen (m_pszSSIDate);

	m_pszSSIDayOfWeek = new char [10 + iEndLen];
	strcpy (m_pszSSIDayOfWeek, "DayOfWeek");
	strcat (m_pszSSIDayOfWeek, m_pszSSIEnd);
	m_iSSIDayOfWeek = strlen (m_pszSSIDayOfWeek);

	m_pszSSIClientIP = new char [9 + iEndLen];
	strcpy (m_pszSSIClientIP, "ClientIP");
	strcat (m_pszSSIClientIP, m_pszSSIEnd);
	m_iSSIClientIP = strlen (m_pszSSIClientIP);

	m_pszSSIClientDomainName = new char [17 + iEndLen];
	strcpy (m_pszSSIClientDomainName, "ClientDomainName");
	strcat (m_pszSSIClientDomainName, m_pszSSIEnd);
	m_iSSIClientDomainName = strlen (m_pszSSIClientDomainName);

	m_pszSSIClientBrowser = new char [14 + iEndLen];
	strcpy (m_pszSSIClientBrowser, "ClientBrowser");
	strcat (m_pszSSIClientBrowser, m_pszSSIEnd);
	m_iSSIClientBrowser = strlen (m_pszSSIClientBrowser);

	m_pszSSIServerIP = new char [9 + iEndLen];
	strcpy (m_pszSSIServerIP, "ServerIP");
	strcat (m_pszSSIServerIP, m_pszSSIEnd);
	m_iSSIServerIP = strlen (m_pszSSIServerIP);

	m_pszSSIServerDomainName = new char [17 + iEndLen];
	strcpy (m_pszSSIServerDomainName, "ServerDomainName");
	strcat (m_pszSSIServerDomainName, m_pszSSIEnd);
	m_iSSIServerDomainName = strlen (m_pszSSIServerDomainName);

	m_pszSSIServerOS = new char [9 + iEndLen];
	strcpy (m_pszSSIServerOS, "ServerOS");
	strcat (m_pszSSIServerOS, m_pszSSIEnd);
	m_iSSIServerOS = strlen (m_pszSSIServerOS);

	m_pszSSIServerVersion = new char [14 + iEndLen];
	strcpy (m_pszSSIServerVersion, "ServerVersion");
	strcat (m_pszSSIServerVersion, m_pszSSIEnd);
	m_iSSIServerVersion = strlen (m_pszSSIServerVersion);

	m_pszSSICounter = new char [8 + iEndLen];
	strcpy (m_pszSSICounter, "Counter");
	strcat (m_pszSSICounter, m_pszSSIEnd);
	m_iSSICounter = strlen (m_pszSSICounter);

	// Open config file
	ConfigFile cfAlajar;
	cfAlajar.Open ("Alajar.conf");

	char* pszRhs;
	if (cfAlajar.GetParameter ("CounterPath", &pszRhs) == OK && pszRhs != NULL) {
		m_pszCounterPath = new char [strlen (pszRhs) + 1];
		strcpy (m_pszCounterPath, pszRhs);
	}
	else _ASSERT (false);

	if (cfAlajar.GetParameter ("LogPath", &pszRhs) == OK && pszRhs != NULL) {
		m_pszLogPath = new char [strlen (pszRhs) + 1];
		strcpy (m_pszLogPath, pszRhs);
	}
	else _ASSERT (false);

	if (cfAlajar.GetParameter ("PageSourcePath", &pszRhs) == OK && pszRhs != NULL) {
		m_pszPageSourcePath = new char [strlen (pszRhs) + 1];
		strcpy (m_pszPageSourcePath, pszRhs);
	}
	else _ASSERT (false);

	if (cfAlajar.GetParameter ("ConfigPath", &pszRhs) == OK && pszRhs != NULL) {
		m_pszConfigPath = new char [strlen (pszRhs) + 1];
		strcpy (m_pszConfigPath, pszRhs);
	}
	else _ASSERT (false);

	if (cfAlajar.GetParameter ("Port", &pszRhs) == OK && pszRhs != NULL) {
		m_siPort = atoi (pszRhs);
	}
	else _ASSERT (false);

	if (cfAlajar.GetParameter ("MaxFileCacheSize", &pszRhs) == OK && pszRhs != NULL) {
		m_pFileCache = new FileCache (atoi (pszRhs) * 1024);
	}
	else _ASSERT (false);

	// Get hostname
	char pszHostName [100];
	gethostname (pszHostName, 100);
	struct hostent* pht = gethostbyname (pszHostName);

	m_pszHostName = new char [strlen (pht->h_name) + 1];
	sprintf (m_pszHostName, pht->h_name);
}


HttpServer::~HttpServer() {

	if (m_pThreadsMutex != NULL) {
		delete m_pThreadsMutex;
	}

	if (m_psz401Text != NULL) {
		delete [] m_psz401Text;
	}

	if (m_psz403Text != NULL) {
		delete [] m_psz403Text;
	}

	if (m_psz404Text != NULL) {
		delete [] m_psz404Text;
	}

	if (m_psz500Text != NULL) {
		delete [] m_psz500Text;
	}

	if (m_psz501Text != NULL) {
		delete [] m_psz501Text;
	}

	if (m_psz505Text != NULL) {
		delete [] m_psz505Text;
	}
	
	if (m_pszServerName != NULL) {
		delete [] m_pszServerName;
	}

	if (m_pFileCache != NULL) {
		delete m_pFileCache;
	}
	
	if (m_pDefaultPageSource != NULL) {
		delete m_pDefaultPageSource;
	}

	if (m_pPageSourceTable != NULL) {
		delete m_pPageSourceTable;
	}

	if (m_pExitMutex != NULL) {
		delete m_pExitMutex;
	}

	if (m_pszSSIBegin != NULL) {
		delete [] m_pszSSIBegin;
	}
	
	if (m_pszSSIEnd != NULL) {
		delete [] m_pszSSIEnd;
	}
	
	if (m_pszSSITime != NULL) {
		delete [] m_pszSSITime;
	}
	
	if (m_pszSSIDate != NULL) {
		delete [] m_pszSSIDate;
	}
	
	if (m_pszSSIDayOfWeek != NULL) {
		delete [] m_pszSSIDayOfWeek;
	}
	
	if (m_pszSSIClientIP != NULL) {
		delete [] m_pszSSIClientIP;
	}
	
	if (m_pszSSIClientDomainName != NULL) {
		delete [] m_pszSSIClientDomainName;
	}
	
	if (m_pszSSIClientBrowser != NULL) {
		delete [] m_pszSSIClientBrowser;
	}
	
	if (m_pszSSIServerIP != NULL) {
		delete [] m_pszSSIServerIP;
	}
	
	if (m_pszSSIServerDomainName != NULL) {
		delete [] m_pszSSIServerDomainName;
	}
	
	if (m_pszSSIServerOS != NULL) {
		delete [] m_pszSSIServerOS;
	}
	
	if (m_pszSSIServerVersion != NULL) {
		delete [] m_pszSSIServerVersion;
	}
	
	if (m_pszSSICounter != NULL) {
		delete [] m_pszSSICounter;
	}

	if (m_pszCounterPath != NULL) {
		delete [] m_pszCounterPath;
	}

	if (m_pszLogPath != NULL) {
		delete [] m_pszLogPath;
	}

	if (m_pszConfigPath != NULL) {
		delete [] m_pszConfigPath;
	}

	if (m_pszPageSourcePath != NULL) {
		delete [] m_pszPageSourcePath;
	}

	if (m_pszHostName != NULL) {
		delete [] m_pszHostName;
	}
}


bool HttpServer::Exit() {

	bool bRetVal;

	m_pExitMutex->WaitForSingleObject();
	bRetVal = m_bExit;
	m_pExitMutex->ReleaseMutex();

	return bRetVal;
}


FileCache* HttpServer::GetFileCache() {
	return m_pFileCache;
}

PageSource* HttpServer::GetDefaultPageSource() {
	return m_pDefaultPageSource;
}

HashTable<PageSource>* HttpServer::GetPageSourceTable() {
	return m_pPageSourceTable;
}

PageSource* HttpServer::GetPageSource (char* pszPageSourceName) {
	return m_pPageSourceTable->GetElement (pszPageSourceName);
}

char* HttpServer::GetServerName() {
	return m_pszServerName;
}

char* HttpServer::Get401Text() {
	return m_psz401Text;
}

char* HttpServer::Get403Text() {
	return m_psz403Text;
}

char* HttpServer::Get404Text() {
	return m_psz404Text;
}

char* HttpServer::Get500Text() {
	return m_psz500Text;
}

char* HttpServer::Get501Text() {
	return m_psz501Text;
}

char* HttpServer::Get505Text() {
	return m_psz505Text;
}

char* HttpServer::GetHostName() {
	return m_pszHostName;
}

char* HttpServer::GetLogPath() {
	return m_pszLogPath;
}

char* HttpServer::GetCounterPath() {
	return m_pszCounterPath;
}

char* HttpServer::GetPageSourcePath() {
	return m_pszPageSourcePath;
}

char* HttpServer::GetConfigPath() {
	return m_pszConfigPath;
}

short HttpServer::GetPort() {
	return m_siPort;
}


///////////////////////
// Server operations //
///////////////////////

void HttpServer::Start() {
	Thread tThread;
	tThread.Start (Run, this);
}

void HttpServer::Stop() {

	m_pExitMutex->WaitForSingleObject();
	m_bExit = true;

	while (m_pssSocket == NULL) {
		Mutex::SpinLock();
	}

	m_pssSocket->Close();
	m_pExitMutex->ReleaseMutex();

	bool bExit, bServing;
	while (true) {

		m_pExitMutex->WaitForSingleObject();
		bExit = m_bExit;
		bServing = m_bServing;
		m_pExitMutex->ReleaseMutex();

		if (!bExit || !bServing) {
			break;
		} else {
			Mutex::SpinLock();
		}
	}
}


/////////////
// Threads //
/////////////

int HttpServer::Run (void* lpvServer) {

	int iErrCode = OK;

	HttpServer* pThis = (HttpServer*) lpvServer;
	Mutex* pThreadsMutex = pThis->m_pThreadsMutex;
	Mutex* pExitMutex = pThis->m_pExitMutex;

	ServerSocket* pssRequest;
	ServerData* pTransport;
	Thread tThread;

	// Initialize exit notification environment
	pThreadsMutex->WaitForSingleObject();
	pThis->m_iNumThreads = 0;
	pThreadsMutex->ReleaseMutex();
	pExitMutex->WaitForSingleObject();
	pThis->m_bExit = false;
	pThis->m_bServing = true;
	pExitMutex->ReleaseMutex();

	// Create a listener socket and open it
	pThis->m_pssSocket = new ServerSocket();
	ServerSocket* pssSocket = pThis->m_pssSocket;
	pssSocket->Open();

	// Configure PageSources
	if (iErrCode = ConfigurePageSources (pThis) != OK) {
		// TODO:  create standard error report scheme
		fprintf (stderr, "\nError configuring PageSources!\n");
		
		pExitMutex->WaitForSingleObject();
		pThis->m_bServing = false;
		pExitMutex->WaitForSingleObject();

		return ERR;
	}

	// Set the socket to listen
	pssSocket->Listen (pThis->GetPort());

	// Loop until we receive the order to finish
	// TODO:  Thread pool
	while (true) {		

		// Accept the incoming connection
		pssRequest = pssSocket->Accept();

		if (pssRequest != NULL) {

			pTransport = new ServerData (pThis, pssRequest);

			// Spawn a thread to handle the request
			iErrCode = tThread.Start (WWWServ, pTransport, THREAD_PRIORITY_NORMAL);

			// Check for thread creation error
			if (iErrCode == ERR) {
				pssRequest->Close();
				delete pssRequest;
				break;
			} else {
				pThreadsMutex->WaitForSingleObject();
				pThis->m_iNumThreads ++;
				pThreadsMutex->ReleaseMutex();
			}
		}
	
		// Check for exit
		if (pThis->Exit()) {
			break;
		}
	}

	// Wait for all remaining threads to exit
	int iNumThreads;
	long lStartTime = Utility::GetTime();
	while (true) {
		pThreadsMutex->WaitForSingleObject();
		iNumThreads = pThis->m_iNumThreads;
		pThreadsMutex->ReleaseMutex();
		
		if (iNumThreads == 0 || Utility::GetTime() - lStartTime > 60) {		// Wait 1 min max
			break;
		}

		Mutex::SpinLock();
	}

	// Close listener socket
	if (pssSocket->Alive()) {
		pssSocket->Close();
	}

	delete pssSocket;

	pExitMutex->WaitForSingleObject();
	pThis->m_bExit = false;
	pThis->m_bServing = false;
	pExitMutex->ReleaseMutex();

	return OK;
}

int HttpServer::WWWServ (void* lpvServer) {

	int iErrCode = OK;

	// Get server and socket
	ServerData* pTransport = (ServerData*) lpvServer;

	HttpServer* pThis = pTransport->GetHttpServer();
	Mutex* pThreadsMutex = pThis->m_pThreadsMutex;
	ServerSocket* pssOpenSocket = pTransport->GetServerSocket();

	// Initialize request handler
	HttpRequest* pHttpRequest = new HttpRequest();
	pTransport->SetHttpRequest (pHttpRequest);
	
	// Parse the request
	iErrCode = pHttpRequest->ParseHeaders (pssOpenSocket, pThis->GetDefaultPageSource(), pThis->GetPageSourceTable());

	if (iErrCode == OK) { 

		// Handle the HTTP request:  /* DEBUG */
		if (pHttpRequest->GetVersion() <= HTTP11) {
			iErrCode = SendHttpResponse (pTransport);
		} else {
			
			while (true) {
				
				// Handle the HTTP request
				if (SendHttpResponse (pTransport) != OK) {
					break;
				}
				
				// Exit loop if no keep-alive
				if (!pHttpRequest->GetKeepAlive() || 
					pHttpRequest->ParseHeaders (pssOpenSocket, pThis->GetDefaultPageSource(), 
						pThis->GetPageSourceTable()) != OK) {
					break;
				}
			}
		}

	} else {

		// Unsupported method?
		if (iErrCode == UNSUPPORTED_HTTP_METHOD) {
			iErrCode = Send501Response (pTransport);
		}

		// Unsupported version
		else if (iErrCode == UNSUPPORTED_HTTP_VERSION) {
			iErrCode = Send505Response (pTransport);
		}
	}

	// Clean up
	pssOpenSocket->Close();
	delete pssOpenSocket;
	delete pHttpRequest;
	delete pTransport;

	pThreadsMutex->WaitForSingleObject();
	pThis->m_iNumThreads --;
	pThreadsMutex->ReleaseMutex();

	return OK;
}


int HttpServer::SendHttpResponse (ServerData* pTransport) {

	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	HttpServer* pThis = pTransport->GetHttpServer();

	// Use Basic Authentication?
	PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();
	if (pPageSource->UseBasicAuthentication()) {

		char* pszLogin;
		char* pszPassword;
		if ((pszLogin = pHttpRequest->GetLogin()) == NULL || (pszPassword = pHttpRequest->GetPassword()) == NULL ||
			!(pPageSource->OnBasicAuthenticate (pszLogin, pszPassword) == OK)) {

			// Respond with a 401
			iErrCode = Send401Response (pTransport);
			return iErrCode;
		}
	}

	// Respond to request by method
	int iMethod = pHttpRequest->GetMethod();
	switch (iMethod) {
	
	case GET:
	case HEAD:

		iErrCode = RespondToGet (pTransport);
		break;

	case POST:

		iErrCode = RespondToPost (pTransport);
		break;

	case TRACE:

		iErrCode = RespondToTrace (pTransport);
		break;

	default:

		// We should never reach here
		return ERR;
	}
	
	// Log
	if (pPageSource->UseLogging()) {

		char* pszTime = Utility::GetTimeStringLocal();
		char* pszIP = pHttpRequest->GetClientIP();
		
		char pszMethod [10];
		if (iMethod == GET) {
			sprintf (pszMethod, "GET");
		} else {
			sprintf (pszMethod, "HEAD");
		}
		
		char* pszUri = pHttpRequest->GetUri();
		
		char pszStatus [10];
		sprintf (pszStatus, "%i", pHttpRequest->GetStatus());
		
		char* pszLogText = new char [strlen (pszTime) + strlen (pszIP) + strlen (pszMethod) + strlen (pszUri) + 
			strlen (pszStatus) + 10];

		sprintf (pszLogText, "%s\t%s\t%s\t%s\t%s\n", pszTime, pszIP, pszMethod, pszUri, pszStatus);
		
		iErrCode = AddToLog (pPageSource->GetName(), pThis->GetLogPath(), pPageSource->UseDailyLogs(), pszLogText);
		
		delete [] pszTime;
		delete [] pszLogText;
	}
	
	return iErrCode;
}


int HttpServer::RespondToGet (ServerData* pTransport) {
	
	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();

	char* pszFileName = pHttpRequest->GetFileName();
	bool bServed = false;

	if (pHttpRequest->GetCached() && pHttpRequest->GetMethod() != HEAD) {

		// Cached already
		iErrCode = SendNotModified (pTransport, pHttpRequest->GetLastModified());

	} else {
	
		// Query the PageSource
		PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();

		// Does the PageSource use overriding and override GET's?
		if (pPageSource->UsePageSource() && pPageSource->OverrideGet()) {
			
			// Create a response object and call the PageSource
			HttpResponse* pHttpResponse = new HttpResponse();
			iErrCode = pPageSource->OnGet (pHttpRequest, pHttpResponse);
			
			if (iErrCode != OK) {
				
				bServed = true;
				iErrCode = Send500Response (pTransport);
			
			} else {
				
				// Did the PageSource return new data?
				if (pHttpResponse->Override()) {
					bServed = true;
					iErrCode = SendPageSourceResponse (pTransport, pHttpResponse);
				}
			}

			pTransport->SetHttpResponse (pHttpResponse);
		}

		// Handle request like traditional server
		if (!bServed) {
			
			// File or directory?
			if (pszFileName [strlen (pszFileName) - 1] == '\\') {
				iErrCode = GetDirectory (pTransport, pszFileName);
			} else {
				iErrCode = GetFile (pTransport, pszFileName, pHttpRequest->GetUri());
			}
		}
	}

	return iErrCode;
}


int HttpServer::GetDirectory (ServerData* pTransport, char* pszDirName) {

	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	FileCache* pFileCache = pTransport->GetHttpServer()->GetFileCache();
	PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();

	bool bServed = false;

	// Eliminate terminal whack
	int iLength = strlen (pszDirName);
	if (pszDirName[iLength - 1] == '\\') {
		pszDirName[iLength - 1] = '\0';
	}
	
	if (pPageSource->GetDefaultFile()) {
		
		char* pszDefaultFile = pPageSource->GetDefaultFile();
		int iDefaultLength = strlen (pszDefaultFile);
		
		// Build path to default file
		char* pszFileName = new char [iLength + iDefaultLength + 2];
		sprintf (pszFileName, pszDirName);
		strcat (pszFileName, "\\");
		strcat (pszFileName, pszDefaultFile);
		
		if (Utility::GetFileAttributes (pszFileName) == FILE_IS_FILE) {
			
			// Build new URI
			char* pszDirUri = pHttpRequest->GetUri();
			char* pszUri = new char [strlen (pszDirUri) + iDefaultLength + 2];
			sprintf (pszUri, pszDirUri);
			strcat (pszUri, pszDefaultFile);
			
			// Treat request as a GET for default file
			bServed = true;
			iErrCode = GetFile (pTransport, pszFileName, pszUri);
			
			delete [] pszFileName;
			delete [] pszUri;
		}
	}
	
	if (!bServed) {
		
		// No default file
		if (pPageSource->AllowDirectoryBrowsing()) {
			
			// Create directory index and send it
			File* pfFile = BuildDirectoryIndex (pszDirName, pHttpRequest->GetUri());
			
			if (pfFile == NULL) {
				iErrCode = Send404Response (pTransport);
			} else {
				// Send directory index
				pfFile->SetMimeType ("text/html");
				iErrCode = SendResponse (200, pTransport, pfFile);

				pfFile->Delete();
				pfFile->Close();
				delete pfFile;
			}

		} else {
			
			// Send not allowed if dir exists
			if (Utility::DoesFileExist (pszDirName)) {
				iErrCode = Send403Response (pTransport);
			} else {
				iErrCode = Send404Response (pTransport);
			}
		}
	}

	return iErrCode;
}


int HttpServer::GetFile (ServerData* pTransport, char* pszFileName, char* pszUri) {
	
	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	FileCache* pFileCache = pTransport->GetHttpServer()->GetFileCache();
	PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();

	char* pszRedirect;

	switch (Utility::GetFileAttributes (pszFileName)) {
		
	case FILE_IS_DIRECTORY:

		// Return redirect to same directory but with a terminal slash
		pszRedirect = new char [strlen (pszUri) + 2];
		sprintf (pszRedirect, pszUri);
		strcat (pszRedirect, "/");
		
		iErrCode = SendRedirect (pTransport, pszRedirect);
		
		delete [] pszRedirect;

		break;

	case FILE_NOT_FOUND:

		iErrCode = Send404Response (pTransport);
		break;
		
	default:
		
		// A file GET has been requested
		File* pfFile = pFileCache->GetFile (pszFileName);
		if (pfFile == NULL) {

			// File not found
			iErrCode = Send404Response (pTransport);

		} else {

			// Run SSI on html files
			if (pPageSource->UseSSI() && stricmp (pfFile->GetMimeType(), "text/html") == 0) {
	
				// Assume that there is enough memory to keep the HTML file in memory
				char* pszNewBuffer;
				iErrCode = RunSSI (pTransport, pszUri, pfFile, &pszNewBuffer);

				if (iErrCode != OK) {
					iErrCode = Send500Response (pTransport);
				} else {
					iErrCode = SendResponse (200, pTransport, "text/html", pszNewBuffer, strlen (pszNewBuffer));
					delete [] pszNewBuffer;
				}

			} else {

				// Send requested file
				iErrCode = SendResponse (200, pTransport, pfFile);
			}

			// Release the file
			pFileCache->ReleaseFile (pfFile);
		}

		break;
	}

	return iErrCode;
}


int HttpServer::RespondToPost (ServerData* pTransport) {

	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();

	// TODO: CGI

	int iNumForms = pHttpRequest->GetNumForms();

	if (pPageSource->UsePageSource() && pPageSource->OverridePost() && iNumForms > 0) {

		// Create a response object
		HttpResponse* pHttpResponse = new HttpResponse();
		
		// Call the pagesource
		iErrCode = pPageSource->OnPost (pHttpRequest, pHttpResponse);
		
		if (iErrCode != OK) {
			iErrCode = Send500Response (pTransport);
		} else {
			iErrCode = SendPageSourceResponse (pTransport, pHttpResponse);
		}
		
		delete pHttpResponse;

	} else {

		// Return standard response:  ignore forms and assume Get of Uri
		iErrCode = RespondToGet (pTransport);
	}

	return iErrCode;
}


int HttpServer::RespondToTrace (ServerData* pTransport) {
	
	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	ServerSocket* pssSocket = pTransport->GetServerSocket();

	char* pszHeaders = pHttpRequest->GetHeaders();

	char pszMessage [] = "200 OK\r\nContent-Type: message/http\r\nContent-Length: ";
	char pszLength [30];
	sprintf (pszLength, "%i\r\n\r\n", strlen (pszHeaders));

	iErrCode = pssSocket->Send (pszMessage);
	iErrCode = pssSocket->Send (pszLength);
	iErrCode = pssSocket->Send (pszHeaders);
	iErrCode = pssSocket->Send ("\r\n");

	pHttpRequest->SetStatus (200);

	return iErrCode;
}


///////////////////
// Send Response //
///////////////////

int HttpServer::SendResponse (int iResponseCode, ServerData* pTransport, File* pfFile) {
	
	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	ServerSocket* pssSocket = pTransport->GetServerSocket();

	// Protocol and status
	char pszHead [] = "HTTP/1.0 ";
	char* pszStatus = GetStatusString (iResponseCode);

	// Date
	char pszDate [] = "\r\nDate: ";
	char* pszDateString = Utility::GetDateStringGMT();

	// Server name
	char pszServer [] = "\r\nServer: ";
	char* pszServerName = pTransport->GetHttpServer()->GetServerName();

	// Content type
	char pszContentType [] = "\r\nContent-Type: ";

	// Content length
	char pszContentLength [512];
	sprintf (pszContentLength, "\r\nContent-Length: %i", pfFile->GetLength());

	// Last modified
	char pszLastMod [] = "\r\nLast Modified: ";
	
	int iNumChars = strlen (pszHead) + strlen (pszStatus) + strlen (pszDate) + strlen (pszDateString) + 
		strlen (pszServer) + strlen (pszServerName) + strlen (pszContentType) + strlen (pszContentLength) + 
		strlen (pszLastMod) + strlen (pfFile->GetLastModifiedTime()) + strlen (pfFile->GetMimeType()) + 10;

	char* pszMessage = new char [iNumChars];
	
	// Protocol and status
	sprintf (pszMessage, pszHead);
	strcat (pszMessage, pszStatus);
	
	// Date
	strcat (pszMessage, pszDate);
	strcat (pszMessage, pszDateString);
	
	// Server name
	strcat (pszMessage, pszServer);
	strcat (pszMessage, pszServerName);
	
	// Content type
	strcat (pszMessage, pszContentType);
	strcat (pszMessage, pfFile->GetMimeType());
	
	// Content length
	strcat (pszMessage, pszContentLength);
	
	// Last Modified
	strcat (pszMessage, pszLastMod);
	strcat (pszMessage, pfFile->GetLastModifiedTime());

	// Send headers
	int iLength = strlen (pszMessage);
	iErrCode = pssSocket->Send (pszMessage, iLength);

	iErrCode += SendCookies (pTransport);

	// End marker
	iErrCode += pssSocket->Send ("\r\n\r\n");

	// Send file
	if (iErrCode == OK && pHttpRequest->GetMethod() != HEAD) {
		
		// Get data from file
		char pszBuffer [MAX_REQUEST_LENGTH];
		void* pVoid = NULL;
		int iBytesRead = MAX_REQUEST_LENGTH;
		
		while (iBytesRead == MAX_REQUEST_LENGTH && iErrCode == OK) {
			iBytesRead = pfFile->ReadData (&pVoid, pszBuffer, MAX_REQUEST_LENGTH);
			iErrCode = pssSocket->Send (pszBuffer, iBytesRead);
		}
	} 
	
	pHttpRequest->SetStatus (iResponseCode);

	delete [] pszStatus;
	delete [] pszDateString;
	delete [] pszMessage;

	return iErrCode;
}


int HttpServer::SendResponse (int iResponseCode, ServerData* pTransport, char* pszMimeType, char* pszData, 
							  int iDataLen) {

	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	ServerSocket* pssSocket = pTransport->GetServerSocket();

	// Protocol and status
	char pszHead [] = "HTTP/1.0 ";
	char* pszStatus = GetStatusString (iResponseCode);

	// Date
	char pszDate [] = "\r\nDate: ";
	char* pszDateString = Utility::GetDateStringGMT();

	// Server name
	char pszServer [] = "\r\nServer: ";
	char* pszServerName = pTransport->GetHttpServer()->GetServerName();

	// Content type
	char pszContentType [] = "\r\nContent-Type: ";

	// Content length
	char pszContentLength [512];
	sprintf (pszContentLength, "\r\nContent-Length: %i", iDataLen);

	// Last modified
	char pszLastMod [] = "\r\nLast Modified: ";

	int iNumChars = strlen (pszHead) + strlen (pszStatus) + strlen (pszDate) + strlen (pszDateString) + 
		strlen (pszServer) + strlen (pszServerName) + strlen (pszContentType) + strlen (pszContentLength) + 
		strlen (pszLastMod) + strlen (pszDateString) + strlen (pszMimeType) + 10;

	char* pszMessage = new char [iNumChars];
	
	// Protocol and status
	sprintf (pszMessage, pszHead);
	strcat (pszMessage, pszStatus);
	
	// Date
	strcat (pszMessage, pszDate);
	strcat (pszMessage, pszDateString);
	
	// Server name
	strcat (pszMessage, pszServer);
	strcat (pszMessage, pszServerName);
	
	// Content type
	strcat (pszMessage, pszContentType);
	strcat (pszMessage, pszMimeType);
	
	// Content length
	strcat (pszMessage, pszContentLength);
	
	// Last Modified
	strcat (pszMessage, pszLastMod);
	strcat (pszMessage, pszDateString);		// Buffers' last-modification date is always "now"

	// Send headers
	int iLength = strlen (pszMessage);
	iErrCode = pssSocket->Send (pszMessage, iLength);

	// Handle cookies
	iErrCode += SendCookies (pTransport);

	// End markers
	iErrCode += pssSocket->Send ("\r\n\r\n");
	
	// Send file
	if (iErrCode == OK && pHttpRequest->GetMethod() != HEAD) {
		iErrCode = pssSocket->Send (pszData, iDataLen);
	} 
	
	pHttpRequest->SetStatus (iResponseCode);

	delete [] pszStatus;
	delete [] pszDateString;
	delete [] pszMessage;

	return iErrCode;
}


int HttpServer::SendRedirect (ServerData* pTransport, char* pszUri) {

	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	ServerSocket* pssSocket = pTransport->GetServerSocket();

	// Protocol and status
	char pszHead [] = "HTTP/1.0 301 Moved Permanently";

	// Date
	char pszDate [] = "\r\nDate: ";
	char* pszDateString = Utility::GetDateStringGMT();

	// Server name
	char pszServer [] = "\r\nServer: ";
	char* pszServerName = pTransport->GetHttpServer()->GetServerName();

	int iNumChars = strlen (pszHead) + strlen (pszDate) + strlen (pszDateString) + 
		strlen (pszServer) + strlen (pszServerName) + strlen (pszUri) + 20;

	char* pszMessage = new char [iNumChars];
	
	// Protocol and status
	sprintf (pszMessage, pszHead);
	
	// Date
	strcat (pszMessage, pszDate);
	strcat (pszMessage, pszDateString);
	
	// Server name
	strcat (pszMessage, pszServer);
	strcat (pszMessage, pszServerName);

	// Location
	strcat (pszMessage, "\r\nLocation: ");
	strcat (pszMessage, pszUri);

	// Send headers
	int iLength = strlen (pszMessage);
	iErrCode = pssSocket->Send (pszMessage, iLength);

	// Handle cookies
	iErrCode += SendCookies (pTransport);

	// End markers
	iErrCode += pssSocket->Send ("\r\n\r\n");
	
	pHttpRequest->SetStatus (301);

	delete [] pszDateString;
	delete [] pszMessage;

	return iErrCode;
}


int HttpServer::SendNotModified (ServerData* pTransport, char* pszLastModified) {

	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	ServerSocket* pssSocket = pTransport->GetServerSocket();

	// Protocol and status
	char pszHead [] = "HTTP/1.0 304 Not Modified";

	// Date
	char pszDate [] = "\r\nDate: ";
	char* pszDateString = Utility::GetDateStringGMT();

	// Server name
	char pszServer [] = "\r\nServer: ";
	char* pszServerName = pTransport->GetHttpServer()->GetServerName();

	int iNumChars = strlen (pszHead) + strlen (pszDate) + strlen (pszDateString) + 
		strlen (pszServer) + strlen (pszServerName) + strlen (pszLastModified) + 30;

	char* pszMessage = new char [iNumChars];
	
	// Protocol and status
	sprintf (pszMessage, pszHead);
	
	// Date
	strcat (pszMessage, pszDate);
	strcat (pszMessage, pszDateString);
	
	// Server name
	strcat (pszMessage, pszServer);
	strcat (pszMessage, pszServerName);
	
	// Location
	strcat (pszMessage, "\r\nLast-Modified: ");
	strcat (pszMessage, pszLastModified);

	// End markers
	strcat (pszMessage, "\r\n\r\n");
	
	// Send headers
	int iLength = strlen (pszMessage);
	iErrCode = pssSocket->Send (pszMessage, iLength);

	pHttpRequest->SetStatus (304);

	delete [] pszDateString;
	delete [] pszMessage;

	return iErrCode;
}


int HttpServer::SendPageSourceResponse (ServerData* pTransport, HttpResponse* pHttpResponse) {

	File* pfFile;
	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	HttpServer* pThis = pTransport->GetHttpServer();
	FileCache* pFileCache = pThis->GetFileCache();
	PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();
	
	switch (pHttpResponse->GetStatus()) {
		
	// OK
	case 200:
		
		if (pHttpResponse->GetType() == RESPONSE_BUFFER) {
			
			// Send a data buffer as a response
			iErrCode = SendResponse (pHttpResponse->GetStatus(), pTransport, pHttpResponse->GetMimeType(), 
				pHttpResponse->GetData(), pHttpResponse->GetLength());

		} else {
			
			// Send a file as a response
			pfFile = pFileCache->GetFile (pHttpResponse->GetFileName());
			if (pfFile == NULL) {
				iErrCode = Send404Response (pTransport);
			} else {
				pfFile->SetMimeType (pHttpResponse->GetMimeType());
				iErrCode = SendResponse (200, pTransport, pfFile);
				pFileCache->ReleaseFile (pfFile);
			}
		}

		break;
		
	case 301:
		
		iErrCode = SendRedirect (pTransport, pHttpResponse->GetRedirectUri());
		break;
		
	case 403:
		
		iErrCode = Send403Response (pTransport);
		break;
		
	case 404:
		
		iErrCode = Send404Response (pTransport);
		break;
		
	case 500:
	default:
		
		iErrCode = Send500Response (pTransport);
		break;
	}

	return iErrCode;
}


int HttpServer::Send401Response (ServerData* pTransport) {
	
	int iErrCode = OK;

	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	ServerSocket* pssSocket = pTransport->GetServerSocket();

	// Protocol and status
	char pszHead [] = "HTTP/1.0 ";
	char* pszStatus = GetStatusString (401);

	// Date
	char pszDate [] = "\r\nDate: ";
	char* pszDateString = Utility::GetDateStringGMT();

	// Server name
	char pszServer [] = "\r\nServer: ";
	char* pszServerName = pTransport->GetHttpServer()->GetServerName();

	// Content type
	char pszContentType [] = "\r\nContent-Type: ";

	int iDataLen;
	char* pszData;
	
	HttpServer* pHttpServer = pTransport->GetHttpServer();
	FileCache* pFileCache = pHttpServer->GetFileCache();
	PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();

	File* pfFile = pFileCache->GetFile (pPageSource->Get401File());
	if (pfFile == NULL) {
		pszData = pHttpServer->Get401Text();
		iDataLen = strlen (pszData);
	} else {
		pszData = NULL;
		iDataLen = pfFile->GetLength();
	}

	// Content length
	char pszContentLength [512];
	sprintf (pszContentLength, "\r\nContent-Length: %i", iDataLen);

	// Last modified
	char pszLastMod [] = "\r\nLast Modified: ";

	// Mime type
	char pszMimeType [] = "text/html";

	// WWW Authenticate
	char pszWWW [] = "WWW-Authenticate: basic realm=\"Alajar\"\r\n";

	int iNumChars = strlen (pszHead) + strlen (pszStatus) + strlen (pszDate) + strlen (pszDateString) + 
		strlen (pszServer) + strlen (pszServerName) + strlen (pszContentType) + strlen (pszContentLength) + 
		strlen (pszLastMod) + strlen (pszDateString) + strlen (pszMimeType) + strlen (pszWWW) + 10;

	char* pszMessage = new char [iNumChars];
	
	// Protocol and status
	sprintf (pszMessage, pszHead);
	strcat (pszMessage, pszStatus);
	
	// Date
	strcat (pszMessage, pszDate);
	strcat (pszMessage, pszDateString);
	
	// Server name
	strcat (pszMessage, pszServer);
	strcat (pszMessage, pszServerName);
	
	// Content type
	strcat (pszMessage, pszContentType);
	strcat (pszMessage, pszMimeType);
	
	// Content length
	strcat (pszMessage, pszContentLength);
	
	// Last Modified
	strcat (pszMessage, pszLastMod);
	strcat (pszMessage, pszDateString);

	// WWW-Authenticate
	strcat (pszMessage, pszWWW);

	// End markers
	strcat (pszMessage, "\r\n\r\n");
	
	// Send headers
	int iLength = strlen (pszMessage);
	iErrCode = pssSocket->Send (pszMessage, iLength);
	
	// Send file
	if (iErrCode == OK && pHttpRequest->GetMethod() != HEAD) {

		if (pszData == NULL) {
			iErrCode = pssSocket->Send (pfFile->GetData(), iDataLen);
			pFileCache->ReleaseFile (pfFile);
		} else {
			iErrCode = pssSocket->Send (pszData, iDataLen);
		}
	}

	pHttpRequest->SetStatus (401);

	delete [] pszStatus;
	delete [] pszDateString;
	delete [] pszMessage;

	return iErrCode;
}


int HttpServer::Send403Response (ServerData* pTransport) {
	
	int iErrCode = OK;
	
	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	HttpServer* pThis = pTransport->GetHttpServer();
	FileCache* pFileCache = pThis->GetFileCache();
	PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();
	
	// Send 403:  try to open default 403 file
	File* pfFile = pFileCache->GetFile (pPageSource->Get403File());
	if (pfFile != NULL) {

		// Send default 403 file
		iErrCode = SendResponse (403, pTransport, pfFile);
		pFileCache->ReleaseFile (pfFile);
	
	} else {
	
		// Send bytes
		iErrCode = SendResponse (403, pTransport, "text/html", pThis->Get403Text(), 
			strlen (pThis->Get403Text()));
	}

	return iErrCode;
}

int HttpServer::Send404Response (ServerData* pTransport) {
	
	int iErrCode = OK;
	
	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	HttpServer* pThis = pTransport->GetHttpServer();
	FileCache* pFileCache = pThis->GetFileCache();
	PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();
	
	// Send 404:  try to open default 404 file
	File* pfFile = pFileCache->GetFile (pPageSource->Get404File());
	if (pfFile != NULL) {
		
		// Send default 404 file
		iErrCode = SendResponse (404, pTransport, pfFile);
		pFileCache->ReleaseFile (pfFile);
		
	} else {
		
		// Send default 404 message
		iErrCode = SendResponse (404, pTransport, "text/html", pThis->Get404Text(), 
			strlen (pThis->Get404Text()));
	}

	return iErrCode;
}


int HttpServer::Send500Response (ServerData* pTransport) {
	
	int iErrCode = OK;
	
	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	HttpServer* pThis = pTransport->GetHttpServer();
	FileCache* pFileCache = pThis->GetFileCache();
	PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();
	
	// Return a 500
	File* pfFile = pFileCache->GetFile (pPageSource->Get500File());
	if (pfFile != NULL) {
		
		// Send default 500 file
		iErrCode = SendResponse (500, pTransport, pfFile);
		pFileCache->ReleaseFile (pfFile);
		
	} else {
		
		// Send bytes
		iErrCode = SendResponse (500, pTransport, "text/html", pThis->Get500Text(), 
			strlen (pThis->Get500Text()));
	}

	return iErrCode;
}


int HttpServer::Send501Response (ServerData* pTransport) {
	
	int iErrCode = OK;
	
	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	HttpServer* pThis = pTransport->GetHttpServer();

	// Send bytes
	iErrCode = SendResponse (501, pTransport, "text/html", pThis->Get501Text(), strlen (pThis->Get501Text()));

	return iErrCode;
}


int HttpServer::Send505Response (ServerData* pTransport) {
	
	int iErrCode = OK;
	
	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
	HttpServer* pThis = pTransport->GetHttpServer();

	// Send bytes
	iErrCode = SendResponse (505, pTransport, "text/html", pThis->Get505Text(), strlen (pThis->Get505Text()));

	return iErrCode;
}

//////////
// Misc //
//////////

char* HttpServer::GetStatusString (int iStatusCode) {
	
	char* pszStatusString;
	
	switch (iStatusCode) {
		
	case 200:

		pszStatusString = new char [7];
		sprintf (pszStatusString, "200 OK");
		break;

	case 304:

		pszStatusString = new char [17];
		sprintf (pszStatusString, "304 Not Modified");
		break;

	case 401:
		pszStatusString = new char [19];
		sprintf (pszStatusString, "401 Not Authorized");
		break;

	case 404:

		pszStatusString = new char [14];
		sprintf (pszStatusString, "404 Not Found");
		break;

	case 500:

		pszStatusString = new char [26];
		sprintf (pszStatusString, "500 Internal Server Error");
		break;

	case 501:

		pszStatusString = new char [21];
		sprintf (pszStatusString, "501 Not Implemented");
		break;

	case 505:

		pszStatusString = new char [32];
		sprintf (pszStatusString, "505 HTTP Version Not Supported");
		break;

	default:

		// Return something weird
		pszStatusString = new char [24];
		sprintf (pszStatusString, "503 Service Unavailable");
		break;
	}

	return pszStatusString;
}

File* HttpServer::BuildDirectoryIndex (char* pszDirName, char* pszUri) {

	char pszTempBuf [512];
	SYSTEMTIME stTime;
	char* pszDate;

	// Build directory search path string
	char* pszDirSearchName = new char [strlen (pszDirName) + 7];
	sprintf (pszDirSearchName, pszDirName);
	strcat (pszDirSearchName, "\\*.*");

	// Get first file
	WIN32_FIND_DATA fdFileData;
	HANDLE hFindFile = ::FindFirstFile (pszDirSearchName, &fdFileData);

	// Does directory exist?
	if (hFindFile == INVALID_HANDLE_VALUE) {
		delete [] pszDirSearchName;
		return NULL;
	}

	// Open tempfile
	TempFile* pTempFile = new TempFile();
	pTempFile->Open();

	/////////////////////////////
	// Write index to tempfile //
	/////////////////////////////

	pTempFile->Write ("<title>Directory index of ");
	pTempFile->Write (pszUri);
	pTempFile->Write ("</title>\n");
	pTempFile->Write ("<h1>Directory index of ");
	pTempFile->Write (pszUri);
	pTempFile->Write ("</h1><p>\n");

	// Get files and directories
	pTempFile->Write ("<table width = \"100%\">\n");

	while (true) {
		
		if (stricmp (fdFileData.cFileName, "..") == 0) {
			pTempFile->Write ("<a href = \"../\">Parent directory</a><p>\n");
			if (!::FindNextFile (hFindFile, &fdFileData)) {
				break;
			}
		}

		if (stricmp (fdFileData.cFileName, ".") == 0) {
			if (!::FindNextFile (hFindFile, &fdFileData)) {
				break;
			}
		
			if (stricmp (fdFileData.cFileName, "..") == 0) {
				pTempFile->Write ("<a href = \"../\">Parent directory</a><p>\n");
				if (!::FindNextFile (hFindFile, &fdFileData)) {
					break;
				}
			}
		}

		pTempFile->Write ("<tr>\n<td><a href = \"./");
		pTempFile->Write (fdFileData.cFileName);
		
		// Dir?
		if ((fdFileData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) == FILE_ATTRIBUTE_DIRECTORY) {
			pTempFile->Write ("/");
			pTempFile->Write ("\">");
			pTempFile->Write (fdFileData.cFileName);
			pTempFile->Write ("/</a></td>\n<td>&nbsp;</td>");
		} else {
			pTempFile->Write ("\">");
			pTempFile->Write (fdFileData.cFileName);
			sprintf (pszTempBuf, "</a></td>\n<td align = \"center\">%i bytes</td>", fdFileData.nFileSizeLow);
			pTempFile->Write (pszTempBuf);
		}

		pTempFile->Write ("<td>");
		
		::FileTimeToSystemTime (&fdFileData.ftLastWriteTime, &stTime);
		pszDate = Utility::GetDateString (&stTime);
		pTempFile->Write (pszDate);
		delete [] pszDate;

		pTempFile->Write ("</td>\n</tr>\n");

		if (!::FindNextFile (hFindFile, &fdFileData)) {
			break;
		}
	}
	pTempFile->Write ("</table>");
	
	::FindClose (hFindFile);

	// Close tempfile
	pTempFile->Close();

	// Open regular file
	File* pfFile = new File();
	pfFile->Open (pTempFile->GetName());

	delete pTempFile;
	delete [] pszDirSearchName;

	return pfFile;
}


int HttpServer::RunSSI (ServerData* pTransport, char* pszUri, File* pfFile, char** ppszNewBuffer) {

	HttpServer* pThis = pTransport->GetHttpServer();
	ServerSocket* pssSocket = pTransport->GetServerSocket();
	HttpRequest* pHttpRequest = pTransport->GetHttpRequest();

	int iFileLen = pfFile->GetLength();
	char* pszBuffer = new char [iFileLen + 1];

	void* vPtr = NULL;
	int iLength = pfFile->ReadData (&vPtr, pszBuffer, iFileLen), iCurLen = 0, iPos = 0, iCounter;
	pszBuffer [iLength] = '\0';

	char* pszEndBuffer = pszBuffer + sizeof (char) * iLength;
	char* pszRemainder = pszBuffer;
	char* pszMarker;
	char* pszTemp;
	char szTemp;
	char* pszReplacement;

	char pszCounter [25];
	bool bReplacement;

	// Loop until entire file is processed
	*ppszNewBuffer = NULL;
	while (true) {

		// Scan for begin marker
		bReplacement = false;
		pszMarker = strstr (pszRemainder, pThis->m_pszSSIBegin);

		if (pszMarker == NULL) {

			// Append remainder of buffer to new buffer and exit
			iCurLen = Utility::AppendString (ppszNewBuffer, pszRemainder, iCurLen);
			break;

		} else {

			// Advance the marker;  exit loop if beyond scope of buffer
			szTemp = pszMarker[0];
			pszReplacement = pszMarker;
			pszMarker[0] = '\0';
			pszMarker += sizeof (char) * pThis->m_iSSIBegin;
			if (pszMarker >= pszEndBuffer) {
				break;
			}

			// Save scanned characters into new buffer
			iCurLen = Utility::AppendString (ppszNewBuffer, pszRemainder, iCurLen);

			// Time
			if (strncmp (pszMarker, pThis->m_pszSSITime, pThis->m_iSSITime) == 0) {
				pszTemp = Utility::GetTimeStringLocal();
				iCurLen = Utility::AppendString (ppszNewBuffer, pszTemp, iCurLen);
				delete [] pszTemp;

				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSITime;
				bReplacement = true;
			}

			// Date
			else if (strncmp (pszMarker, pThis->m_pszSSIDate, pThis->m_iSSIDate) == 0) {
				pszTemp = Utility::GetDateStringLocal();
				iCurLen = Utility::AppendString (ppszNewBuffer, pszTemp, iCurLen);
				delete [] pszTemp;

				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSIDate;
				bReplacement = true;
			}

			// Counter
			else if (strncmp (pszMarker, pThis->m_pszSSICounter, pThis->m_iSSICounter) == 0) {

				iCounter = RunSSICounter (pThis->GetCounterPath(), pszUri);
				sprintf (pszCounter, "%i", iCounter);
				iCurLen = Utility::AppendString (ppszNewBuffer, pszCounter, iCurLen);
				
				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSICounter;
				bReplacement = true;
			}

			// DayOfWeek
			else if (strncmp (pszMarker, pThis->m_pszSSIDayOfWeek, pThis->m_iSSIDayOfWeek) == 0) {

				char* pszDayOfWeek = Utility::GetDayOfWeekString();
				iCurLen = Utility::AppendString (ppszNewBuffer, pszDayOfWeek, iCurLen);
				delete [] pszDayOfWeek;

				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSIDayOfWeek;
				bReplacement = true;
			}

			// ClientIP
			else if (strncmp (pszMarker, pThis->m_pszSSIClientIP, pThis->m_iSSIClientIP) == 0) {

				iCurLen = Utility::AppendString (ppszNewBuffer, pssSocket->GetClientIP(), iCurLen);

				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSIClientIP;
				bReplacement = true;
			}

			// ClientDomainName
			else if (strncmp (pszMarker, pThis->m_pszSSIClientDomainName, pThis->m_iSSIClientDomainName) == 0) {

				iCurLen = Utility::AppendString (ppszNewBuffer, pssSocket->GetClientDomainName(), iCurLen);

				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSIClientDomainName;
				bReplacement = true;
			}

			// ClientBrowser
			else if (strncmp (pszMarker, pThis->m_pszSSIClientBrowser, pThis->m_iSSIClientBrowser) == 0) {

				iCurLen = Utility::AppendString (ppszNewBuffer, pHttpRequest->GetBrowserName(), iCurLen);

				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSIClientBrowser;
				bReplacement = true;
			}

			// ServerDomainName
			else if (strncmp (pszMarker, pThis->m_pszSSIServerDomainName, pThis->m_iSSIServerDomainName) == 0) {

				iCurLen = Utility::AppendString (ppszNewBuffer, pThis->GetHostName(), iCurLen);

				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSIServerDomainName;
				bReplacement = true;
			}

			// ServerOS
			else if (strncmp (pszMarker, pThis->m_pszSSIServerOS, pThis->m_iSSIServerOS) == 0) {

				char* pszVer = Utility::GetOSVersion();
				iCurLen = Utility::AppendString (ppszNewBuffer, pszVer, iCurLen);
				delete [] pszVer;

				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSIServerOS;
				bReplacement = true;
			}

			// ServerVersion
			else if (strncmp (pszMarker, pThis->m_pszSSIServerVersion, pThis->m_iSSIServerVersion) == 0) {

				iCurLen = Utility::AppendString (ppszNewBuffer, pThis->GetServerName(), iCurLen);

				pszRemainder = pszMarker += sizeof (char) * pThis->m_iSSIServerVersion;
				bReplacement = true;
			}

			// No match
			else {
				iCurLen = Utility::AppendString (ppszNewBuffer, pThis->m_pszSSIBegin, iCurLen);
				pszRemainder = pszMarker;
			}

			pszReplacement[0] = szTemp;
		}
	}

	delete [] pszBuffer;
	
	return OK;
}

int HttpServer::RunSSICounter (char* pszCounterPath, char* pszUri) {
	
	char* pszFurther = Utility::SlashToUnderscore (pszUri);
	char* pszCounterFile = new char [strlen (pszCounterPath) + strlen (pszFurther) + 7];
	sprintf (pszCounterFile, pszCounterPath);
	strcat (pszCounterFile, "\\");
	strcat (pszCounterFile, pszFurther);
	strcat (pszCounterFile, ".txt");
	
	// Open file for exclusive access
	filebuf fbFile;
	while (true) {
		fbFile.open (pszCounterFile, ios::in | ios::out, filebuf::sh_none);
		if (fbFile.is_open() == 0) {
			Mutex::SpinLock();
		} else {
			break;
		}
	}
	
	// Read contents of file
	struct _stat stFile;
	_stat (pszCounterFile, &stFile);
	int iFileLength = stFile.st_size;
	
	char* pszBuffer = new char [iFileLength + 3];
	int iCounter, iNumBytes = fbFile.sgetn (pszBuffer, iFileLength);
	
	if (iNumBytes == 0) {
		iCounter = 0;
	} else {
		sscanf (pszBuffer, "%i", &iCounter);
	}
	
	// Write counter + 1 to file and close file
	iCounter ++;
	sprintf (pszBuffer, "%i", iCounter);
	
	fbFile.seekpos (0, ios::out);
	fbFile.sputn (pszBuffer, strlen (pszBuffer));
	fbFile.close();

	delete [] pszFurther;
	delete [] pszBuffer;
	delete [] pszCounterFile;

	return iCounter;
}

int HttpServer::AddToLog (char* pszPageSource, char* pszLogPath, bool bDailyLog, char* pszText) {

	if (pszText == NULL || pszLogPath == NULL || pszPageSource == NULL) {
		return ERR;
	}

	char* pszFileName = NULL;
	if (stricmp (pszPageSource, "Default") != 0) {
		pszFileName = new char [strlen (pszLogPath) + strlen (pszPageSource) + 50];
		sprintf (pszFileName, pszLogPath);
		strcat (pszFileName, "\\");
		strcat (pszFileName, pszPageSource);
	} else {
		pszFileName = new char [strlen (pszLogPath) + 50];
		sprintf (pszFileName, pszLogPath);
	}

	if (bDailyLog) {
		const long lTime = time (NULL);
		struct tm* tmTime = localtime (&lTime);

		char pszName [50];
		sprintf (pszName, "\\Alajar_%i_%i_%i.log", tmTime->tm_mon + 1, tmTime->tm_mday, tmTime->tm_year + 1900);
		strcat (pszFileName, pszName);

	} else {
		strcat (pszFileName, "\\Alajar.log");
	}

	// Open log for exclusive append use
	filebuf fbFile;
	while (true) {
		fbFile.open (pszFileName, ios::app, filebuf::sh_none);
		if (fbFile.is_open() == 0) {
			Mutex::SpinLock();
		} else {
			break;
		}
	}

	int iLength = strlen (pszText);
	int iErrCode = fbFile.sputn (pszText, iLength);

	fbFile.close();

	delete [] pszFileName;

	return (iErrCode == iLength) ? OK:ERR;
}


int HttpServer::SendCookies (ServerData* pTransport) {

	int i, iErrCode = OK;

	HttpResponse* pHttpResponse = pTransport->GetHttpResponse();
	
	if (pHttpResponse != NULL) {

		HttpRequest* pHttpRequest = pTransport->GetHttpRequest();
		ServerSocket* pssSocket = pTransport->GetServerSocket();

		int iCookiesSet, iCookiesDeleted;
		char* pszCookieText;
		char* pszExpirationDate;

		char** ppszCookieName;
		char** ppszCookieValue;
		int* piTTLinSeconds;
		char** ppszCookiePath;

		time_t lTime;

		if ((iCookiesSet = pHttpResponse->GetNumCookiesSet()) > 0 || 
			(iCookiesDeleted = pHttpResponse->GetNumCookiesDeleted())) {

			PageSource* pPageSource = (PageSource*) pHttpRequest->GetPageSource();

			if (iCookiesSet > 0) {

				pHttpResponse->GetCookiesSet (&ppszCookieName, &ppszCookieValue, &piTTLinSeconds, &ppszCookiePath);
				
				for (i = 0; i < iCookiesSet; i ++) {
				
					lTime = time (NULL) + piTTLinSeconds[i];
					pszExpirationDate = ctime (&lTime);
					pszExpirationDate [strlen (pszExpirationDate)] = '\0';

					pszCookieText = new char [strlen (ppszCookieName[i]) + strlen (ppszCookieValue[i]) + 
						strlen (pszExpirationDate) + strlen (pPageSource->GetName()) + 
						strlen (ppszCookiePath[i]) + 100];

					sprintf (pszCookieText, "\r\nSet-Cookie: ");
					strcat (pszCookieText, ppszCookieName[i]);
					strcat (pszCookieText, "=");
					strcat (pszCookieText, ppszCookieValue[i]);
					strcat (pszCookieText, "; expires=");
					strcat (pszCookieText, pszExpirationDate);
					strcat (pszCookieText, " GMT; path=/");
					strcat (pszCookieText, pPageSource->GetName());
					strcat (pszCookieText, ppszCookiePath[i]);

					iErrCode = pssSocket->Send (pszCookieText);

					delete [] pszCookieText;
				}
			}

			if (iCookiesDeleted > 0) {
				
				pHttpResponse->GetCookiesDeleted (&ppszCookieName, &ppszCookiePath);
				
				for (i = 0; i < iCookiesSet; i ++) {
				
					lTime = time (NULL) - 10;
					pszExpirationDate = ctime (&lTime);
					pszExpirationDate [strlen (pszExpirationDate)] = '\0';

					pszCookieText = new char [strlen (ppszCookieName[i]) + strlen (pszExpirationDate) + 
						strlen (pPageSource->GetName()) + strlen (ppszCookiePath[i]) + 100];

					sprintf (pszCookieText, "\r\nSet-Cookie: ");
					strcat (pszCookieText, ppszCookieName[i]);
					strcat (pszCookieText, "=null");
					strcat (pszCookieText, "; expires=");
					strcat (pszCookieText, pszExpirationDate);
					strcat (pszCookieText, " GMT; path=/");
					strcat (pszCookieText, pPageSource->GetName());
					strcat (pszCookieText, ppszCookiePath[i]);

					iErrCode = pssSocket->Send (pszCookieText);

					delete [] pszCookieText;
				}
			}
		}
	}

	return iErrCode;
}

int HttpServer::ConfigurePageSources (HttpServer* pThis) {

	int i, iNumPageSources, iErrCode = OK;

	// Clean up previous PageSources if necessary
	if (pThis->m_pPageSourceTable != NULL) {
		delete pThis->m_pPageSourceTable;
	}

	if (pThis->m_pDefaultPageSource != NULL) {
		delete pThis->m_pDefaultPageSource;
	}

	// Read config files
	char* pszConfigPath = pThis->GetConfigPath();
	char* pszTemp = new char [strlen (pszConfigPath) + 8];
	strcpy (pszTemp, pszConfigPath);
	strcat (pszTemp, "\\*.conf");

	char** ppszFileName = NULL;
	File::GetAllFileNames (pszTemp, &iNumPageSources, &ppszFileName);

	delete [] pszTemp;

	// Initialize pagesource table
	pThis->m_pPageSourceTable = new HashTable<PageSource>(iNumPageSources);	

	///////////////////////////////////
	// Get data for all page sources //
	///////////////////////////////////

	ConfigFile cfPageSource;
	PageSource* pPageSource;
	char* pszRhs;
	char* pszRhs2;
	char* pszName;
	char* pszLibName;
	char* pszPageSourcePath = pThis->GetPageSourcePath();
	int iPageSourcePathLen = strlen (pszPageSourcePath);

	for (i = 0; i < iNumPageSources; i ++) {

		// Open PageSource config file
		pszTemp = new char [strlen (pszConfigPath) + strlen (ppszFileName[i]) + 2];
		strcpy (pszTemp, pszConfigPath);
		strcat (pszTemp, "\\");
		strcat (pszTemp, ppszFileName[i]);
		cfPageSource.Open (pszTemp);
		delete [] pszTemp;
		
		// Create new pagesource
		pPageSource = new PageSource();
		
		// Name
		if (cfPageSource.GetParameter ("Name", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetName (pszRhs);
			pszName = pszRhs;
		}
		else _ASSERT (false);

		// 401File
		if (cfPageSource.GetParameter ("401File", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->Set401File (pszRhs);
		}
		else _ASSERT (false);
		
		// 403File
		if (cfPageSource.GetParameter ("403File", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->Set403File (pszRhs);
		}
		else _ASSERT (false);
		
		// 404File
		if (cfPageSource.GetParameter ("404File", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->Set404File (pszRhs);
		}
		else _ASSERT (false);
		
		// 500File
		if (cfPageSource.GetParameter ("404File", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->Set404File (pszRhs);
		}
		else _ASSERT (false);
		
		// 501File
		if (cfPageSource.GetParameter ("404File", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->Set404File (pszRhs);
		}
		else _ASSERT (false);
		
		// AllowDirectoryBrowsing
		if (cfPageSource.GetParameter ("AllowDirectoryBrowsing", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetAllowDirectoryBrowsing (atoi (pszRhs) == 1);
		}
		else _ASSERT (false);

		// BasePath
		if (cfPageSource.GetParameter ("BasePath", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetBasePath (pszRhs);
		}
		else _ASSERT (false);
		
		// DefaultFile
		if (cfPageSource.GetParameter ("DefaultFile", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetDefaultFile (pszRhs);
		}
		else _ASSERT (false);
		
		// OverrideGet
		if (cfPageSource.GetParameter ("OverrideGet", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetOverrideGet (atoi (pszRhs) == 1);
		}
		else _ASSERT (false);
		
		// OverridePost
		if (cfPageSource.GetParameter ("OverridePost", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetOverridePost (atoi (pszRhs) == 1);
		}
		else _ASSERT (false);
		
		// UseBasicAuthentication
		if (cfPageSource.GetParameter ("UseBasicAuthentication", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetUseBasicAuthentication (atoi (pszRhs) == 1);
		}
		else _ASSERT (false);

		// UseDefaultFile
		if (cfPageSource.GetParameter ("UseDefaultFile", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetUseDefaultFile (atoi (pszRhs) == 1);
		}
		else _ASSERT (false);
		
		// UseSSI
		if (cfPageSource.GetParameter ("UseSSI", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetUseSSI (atoi (pszRhs) == 1);
		}
		else _ASSERT (false);
		
		// UseLogging
		if (cfPageSource.GetParameter ("UseLogging", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetUseLogging (atoi (pszRhs) == 1);
		}
		else _ASSERT (false);
		
		// UseDailyLogs
		if (cfPageSource.GetParameter ("UseDailyLogs", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetUseDailyLogs (atoi (pszRhs) == 1);
		}
		else _ASSERT (false);
		
		// UsePageSource
		if (cfPageSource.GetParameter ("UsePageSource", &pszRhs) == OK && pszRhs != NULL) {
			pPageSource->SetUsePageSource (atoi (pszRhs) == 1);
		}
		else _ASSERT (false);
		
		// PageSourceLibrary, PageSourceNameSpace
		if (atoi (pszRhs) == 1) {

			if (cfPageSource.GetParameter ("PageSourceLibrary", &pszRhs) != OK || pszRhs == NULL) _ASSERT (false);
			if (cfPageSource.GetParameter ("PageSourceNameSpace", &pszRhs2) != OK || pszRhs2 == NULL) _ASSERT (false);
			
			pszLibName = new char [iPageSourcePathLen + strlen (pszRhs) + 6];
			sprintf (pszLibName, pszPageSourcePath);
			strcat (pszLibName, "\\");
			strcat (pszLibName, pszRhs);
			strcat (pszLibName, ".dll");
			
			iErrCode = pPageSource->LoadLibrary (pszLibName, pszRhs2);

			// TODO: raise error on failure
			
			delete [] pszLibName;
		}
		
		// Add to table if everything went OK
		if (stricmp (pszName, "Default") == 0) {
			pThis->m_pDefaultPageSource = pPageSource;
		} else {
			pThis->m_pPageSourceTable->AddElement (pszName, pPageSource);
		}

		cfPageSource.Close();
	}

	if (iNumPageSources > 0) {
		for (i = 0; i <  iNumPageSources; i ++) {
			delete [] ppszFileName[i];
		}

		delete [] ppszFileName;
	}

	return iErrCode;
}