// HttpServer.h: interface for the HttpServer class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFXServerData_H__7E3E4053_A569_11D1_9C4E_0060083E8062__INCLUDED_)
#define AFXServerData_H__7E3E4053_A569_11D1_9C4E_0060083E8062__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "ServerSocket.h"
#include "HttpRequest.h"
#include "FileCache.h"
#include "PageSource.h"
#include "HttpResponse.h"

class ServerData;
class HttpServer {

	friend class HttpRequest;
	friend class HttpResponse;

private:

	// True if closing server
	bool m_bExit;

	// True if still serving
	bool m_bServing;

	// Host name
	char* m_pszHostName;

	// Listener port
	short m_siPort;

	// Listener socket
	ServerSocket* m_pssSocket;

	// Mutex for exit
	Mutex* m_pExitMutex;

	// Mutex for thread count
	Mutex* m_pThreadsMutex;

	// Current number of threads
	int m_iNumThreads;

	// Name of server
	char* m_pszServerName;

	// Default error strings
	char* m_psz401Text;
	char* m_psz403Text;
	char* m_psz404Text;
	char* m_psz500Text;
	char* m_psz501Text;
	char* m_psz505Text;

	// SSI
	char* m_pszSSIBegin;
	char* m_pszSSIEnd;
	char* m_pszSSITime;
	char* m_pszSSIDate;
	char* m_pszSSIDayOfWeek;
	char* m_pszSSIClientIP;
	char* m_pszSSIClientDomainName;
	char* m_pszSSIClientBrowser;
	char* m_pszSSIServerIP;
	char* m_pszSSIServerDomainName;
	char* m_pszSSIServerOS;
	char* m_pszSSIServerVersion;
	char* m_pszSSICounter;
	int m_iSSIBegin;
	int m_iSSIEnd;
	int m_iSSITime;
	int m_iSSIDate;
	int m_iSSIDayOfWeek;
	int m_iSSIClientIP;
	int m_iSSIClientDomainName;
	int m_iSSIClientBrowser;
	int m_iSSIServerIP;
	int m_iSSIServerDomainName;
	int m_iSSIServerOS;
	int m_iSSIServerVersion;
	int m_iSSICounter;

	// Paths
	char* m_pszCounterPath;
	char* m_pszLogPath;
	char* m_pszPageSourcePath;
	char* m_pszConfigPath;

	// Filecache
	FileCache* m_pFileCache;

	// Page sources
	PageSource* m_pDefaultPageSource;
	HashTable<PageSource>* m_pPageSourceTable;

	static int Run (void* lpvServer);
	static int ConfigurePageSources (HttpServer* pThis);
	virtual bool Exit();

	virtual inline FileCache* GetFileCache();
	virtual inline PageSource* GetDefaultPageSource();
	virtual inline HashTable<PageSource>* GetPageSourceTable();
	virtual inline PageSource* GetPageSource (char* pszPageSourceName);
	virtual inline char* GetServerName();
	virtual inline char* Get401Text();
	virtual inline char* Get403Text();
	virtual inline char* Get404Text();
	virtual inline char* Get500Text();
	virtual inline char* Get501Text();
	virtual inline char* Get505Text();
	virtual inline char* GetHostName();
	virtual inline char* GetLogPath();
	virtual inline char* GetCounterPath();
	virtual inline char* GetPageSourcePath();
	virtual inline char* GetConfigPath();
	virtual inline short GetPort();
	
	static int WWWServ (void* lpvSocket);
	static int SendHttpResponse (ServerData* pTransport);

	static int RespondToGet (ServerData* pTransport);
	static int GetDirectory (ServerData* pTransport, char* pszDirName);
	static int GetFile (ServerData* pTransport, char* pszFileName, char* pszUri);

	static int RespondToPost (ServerData* pTransport);
	static int RespondToTrace (ServerData* pTransport);

	static int SendResponse (int iResponseCode, ServerData* pTransport, File* pfFile);
	static int SendResponse (int iResponseCode, ServerData* pTransport, char* pszMimeType, char* pszData, 
							 int iDataLen);

	static int SendRedirect (ServerData* pTransport, char* pszUri);
	static int SendNotModified (ServerData* pTransport, char* pszLastModified);
	static int SendPageSourceResponse (ServerData* pTransport, HttpResponse* pHttpResponse);

	static int Send401Response (ServerData* pTransport);
	static int Send403Response (ServerData* pTransport);
	static int Send404Response (ServerData* pTransport);
	static int Send500Response (ServerData* pTransport);
	static int Send501Response (ServerData* pTransport);
	static int Send505Response (ServerData* pTransport);

	static inline char* GetStatusString (int iStatusCode);
	static inline File* BuildDirectoryIndex (char* pszDirName, char* pszUri);
	static int RunSSI (ServerData* pTransport, char* pszUri, File* pfFile, char** ppszNewBuffer);
	static int RunSSICounter (char* pszCounterPath, char* pszUri);

	static int AddToLog (char* pszPageSource, char* pszLogPath, bool bDailyLog, char* pszText);
	static int SendCookies (ServerData* pTransport);

public:

	HttpServer();
	virtual ~HttpServer();

	virtual void Start();
	virtual void Stop();
};


class ServerData {

public:

	ServerData (HttpServer* pHttpServer, ServerSocket* pServerSocket) {
		m_pHttpServer = pHttpServer;
		m_pServerSocket = pServerSocket;
		m_pHttpRequest = NULL;
		m_pHttpResponse = NULL;
	}

	virtual ~ServerData() {
		if (m_pHttpResponse != NULL) {
			delete m_pHttpResponse;
		}
	}

	virtual inline HttpServer* ServerData::GetHttpServer() {
		return m_pHttpServer;
	}

	virtual inline ServerSocket* ServerData::GetServerSocket() {
		return m_pServerSocket;
	}

	virtual inline HttpRequest* ServerData::GetHttpRequest() {
		return m_pHttpRequest;
	}

	virtual inline HttpResponse* ServerData::GetHttpResponse() {
		return m_pHttpResponse;
	}

	virtual inline void ServerData::SetHttpRequest (HttpRequest* pHttpRequest) {
		m_pHttpRequest = pHttpRequest;
	}

	virtual inline void ServerData::SetHttpResponse (HttpResponse* pHttpResponse) {
		m_pHttpResponse = pHttpResponse;
	}

private:

	HttpServer* m_pHttpServer;
	ServerSocket* m_pServerSocket;
	HttpRequest* m_pHttpRequest;
	HttpResponse* m_pHttpResponse;
};


#endif // !defined(AFXServerData_H__7E3E4053_A569_11D1_9C4E_0060083E8062__INCLUDED_)