// PageSource.cpp: implementation of the PageSource class.
//
//////////////////////////////////////////////////////////////////////

#include "PageSource.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

PageSource::PageSource() {

	m_pszName = NULL;

	m_psz401File = NULL;
	m_psz403File = NULL;
	m_psz404File = NULL;
	m_psz500File = NULL;
	m_psz501File = NULL;
	
	m_bBrowsingAllowed = false;
	m_bDefaultFile = false;
	m_bBasicAuthentication = false;
	m_bOverrideGet = false;
	m_bOverridePost = false;
	m_bUseSSI = false;
	m_bUsePageSource = false;
	m_bUseLogging = false;
	m_bUseDailyLogs = false;

	m_pszDefaultFile = NULL;
	m_pszBasePath = NULL;

	m_pfxnOnInitialize = NULL;
	m_pfxnOnBasicAuthenticate = NULL;
	m_pfxnOnGet = NULL;
	m_pfxnOnPost = NULL;
	m_pfxnOnFinalize = NULL;

	m_hPageSource = NULL;
}

PageSource::~PageSource() {

	if (m_bUsePageSource) {
		m_pfxnOnFinalize();
	}

	if (m_hPageSource != NULL) {
		::FreeLibrary (m_hPageSource);
	}

	if (m_pszName != NULL) {
		delete [] m_pszName;
	}

	if (m_psz401File != NULL) {
		delete [] m_psz401File;
	}

	if (m_psz403File != NULL) {
		delete [] m_psz403File;
	}

	if (m_psz404File != NULL) {
		delete [] m_psz404File;
	}

	if (m_psz500File != NULL) {
		delete [] m_psz500File;
	}

	if (m_psz501File != NULL) {
		delete [] m_psz501File;
	}

	if (m_pszDefaultFile != NULL) {
		delete [] m_pszDefaultFile;
	}

	if (m_pszBasePath != NULL) {
		delete [] m_pszBasePath;
	}
}


///////////////
// Configure //
///////////////

void PageSource::SetName (char* pszName) {
	if (pszName != NULL) {
		m_pszName = new char [strlen (pszName) + 1];
		strcpy (m_pszName, pszName);
	}
}

int PageSource::LoadLibrary (char* pszLibraryName, char* pszNameSpace) {

	if (pszLibraryName == NULL || pszNameSpace == NULL) {
		return ERR;
	}

	m_hPageSource = ::LoadLibrary (pszLibraryName);
	if (m_hPageSource == NULL) {
		return ERR;
	}
	
	char* pszFunctionName = new char [strlen (pszNameSpace) + 25];
	
	// Get address for OnInitialize
	strcpy (pszFunctionName, pszNameSpace);
	strcat (pszFunctionName, "_OnInitialize");
	m_pfxnOnInitialize = (Fxn_Int) ::GetProcAddress (m_hPageSource, pszFunctionName);

	if (m_pfxnOnInitialize == NULL) {
		delete [] pszFunctionName;
		return ERR;
	}

	// Get address for OnBasicAuthenticate
	strcpy (pszFunctionName, pszNameSpace);
	strcat (pszFunctionName, "_OnBasicAuthenticate");
	m_pfxnOnBasicAuthenticate = (Fxn_Int_pChar_pChar) ::GetProcAddress (m_hPageSource, pszFunctionName);

	if (m_pfxnOnBasicAuthenticate == NULL) {
		delete [] pszFunctionName;
		return ERR;
	}
	
	// Get address for OnGet
	strcpy (pszFunctionName, pszNameSpace);
	strcat (pszFunctionName, "_OnGet");
	m_pfxnOnGet = (Fxn_Int_pHttpReq_pHttpResp) ::GetProcAddress (m_hPageSource, pszFunctionName);
	
	if (m_pfxnOnGet == NULL) {
		delete [] pszFunctionName;
		return ERR;
	}
	
	// Get address for OnPost
	strcpy (pszFunctionName, pszNameSpace);
	strcat (pszFunctionName, "_OnPost");
	m_pfxnOnPost = (Fxn_Int_pHttpReq_pHttpResp) ::GetProcAddress (m_hPageSource, pszFunctionName);
	
	if (m_pfxnOnPost == NULL) {
		return ERR;
	}

	// Get address for OnFinalize
	strcpy (pszFunctionName, pszNameSpace);
	strcat (pszFunctionName, "_OnFinalize");
	m_pfxnOnFinalize = (Fxn_Int) ::GetProcAddress (m_hPageSource, pszFunctionName);

	delete [] pszFunctionName;

	if (m_pfxnOnFinalize == NULL) {
		return ERR;
	}

	// Initialize PageSource
	return m_pfxnOnInitialize();
}

void PageSource::SetBasePath (char* pszBasePath) {
	if (pszBasePath != NULL) {
		m_pszBasePath = new char [strlen (pszBasePath) + 1];
		strcpy (m_pszBasePath, pszBasePath);
	}
}

void PageSource::SetDefaultFile (char* pszDefaultFile) {
	if (pszDefaultFile != NULL) {
		m_pszDefaultFile = new char [strlen (pszDefaultFile) + 1];
		strcpy (m_pszDefaultFile, pszDefaultFile);
	}
}

void PageSource::Set401File (char* psz401File) {
	if (psz401File != NULL) {
		m_psz401File = new char [strlen (psz401File) + 1];
		strcpy (m_psz401File, psz401File);
	}
}

void PageSource::Set403File (char* psz403File) {
	if (psz403File != NULL) {
		m_psz403File = new char [strlen (psz403File) + 1];
		strcpy (m_psz403File, psz403File);
	}
}

void PageSource::Set404File (char* psz404File) {
	if (psz404File != NULL) {
		m_psz404File = new char [strlen (psz404File) + 1];
		strcpy (m_psz404File, psz404File);
	}
}

void PageSource::Set500File (char* psz500File) {
	if (psz500File != NULL) {
		m_psz500File = new char [strlen (psz500File) + 1];
		strcpy (m_psz500File, psz500File);
	}
}

void PageSource::Set501File (char* psz501File) {
	if (psz501File != NULL) {
		m_psz501File = new char [strlen (psz501File) + 1];
		strcpy (m_psz501File, psz501File);
	}
}

void PageSource::SetAllowDirectoryBrowsing (bool bAllowDirectoryBrowsing) {
	m_bBrowsingAllowed = bAllowDirectoryBrowsing;
}

void PageSource::SetUseDefaultFile (bool bUseDefaultFile) {
	m_bDefaultFile = bUseDefaultFile;
}

void PageSource::SetUseBasicAuthentication (bool bUseBasicAuthentication) {
	m_bBasicAuthentication = bUseBasicAuthentication;
}

void PageSource::SetOverrideGet (bool bOverrideGet) {
	m_bOverrideGet = bOverrideGet;
}

void PageSource::SetOverridePost(bool bOverridePost) {
	m_bOverridePost = bOverridePost;
}

void PageSource::SetUseSSI (bool bUseSSI) {
	m_bUseSSI = bUseSSI;
}

void PageSource::SetUsePageSource (bool bUsePageSource) {
	m_bUsePageSource = bUsePageSource;
}

void PageSource::SetUseLogging (bool bUseLogging) {
	m_bUseLogging = bUseLogging;
}

void PageSource::SetUseDailyLogs (bool bUseDailyLogs) {
	m_bUseDailyLogs = bUseDailyLogs;
}


////////////////////////
// Configuration data //
////////////////////////

char* PageSource::GetName() {
	return m_pszName;
}

char* PageSource::GetBasePath() {
	return m_pszBasePath;
}

char* PageSource::GetDefaultFile() {
	return m_pszDefaultFile;
}

char* PageSource::Get401File() {
	return m_psz401File;
}

char* PageSource::Get403File() {
	return m_psz403File;
}

char* PageSource::Get404File() {
	return m_psz404File;
}

char* PageSource::Get500File() {
	return m_psz500File;
}

char* PageSource::Get501File() {
	return m_psz501File;
}

bool PageSource::AllowDirectoryBrowsing() {
	return m_bBrowsingAllowed;
}

bool PageSource::UseDefaultFile() {
	return m_bDefaultFile;
}

bool PageSource::UseBasicAuthentication() {
	return m_bBasicAuthentication;
}

bool PageSource::OverrideGet() {
	return m_bOverrideGet;
}

bool PageSource::OverridePost() {
	return m_bOverridePost;
}

bool PageSource::UsePageSource() {
	return m_bUsePageSource;
}

bool PageSource::UseSSI() {
	return m_bUseSSI;
}

bool PageSource::UseLogging() {
	return m_bUseLogging;
}

bool PageSource::UseDailyLogs() {
	return m_bUseDailyLogs;
}


/////////////
// Methods //
/////////////

int PageSource::OnBasicAuthenticate (char* pszLogin, char* pszPassword) {
	return m_pfxnOnBasicAuthenticate (pszLogin, pszPassword);
}

int PageSource::OnGet (HttpRequest* pHttpRequest, HttpResponse* pHttpResponse) {
	return m_pfxnOnGet (pHttpRequest, pHttpResponse);
}

int PageSource::OnPost (HttpRequest* pHttpRequest, HttpResponse* pHttpResponse) {
	return m_pfxnOnPost (pHttpRequest, pHttpResponse);
}
