// PageSource.h: interface for the PageSource class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_PAGESOURCE_H__1F955CC5_AD9D_11D1_9C61_0060083E8062__INCLUDED_)
#define AFX_PAGESOURCE_H__1F955CC5_AD9D_11D1_9C61_0060083E8062__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "HttpRequest.h"
#include "HttpResponse.h"

typedef int (*Fxn_Int_pChar_pChar)(char*, char*);
typedef int (*Fxn_Int) (void);
typedef int (*Fxn_Int_pHttpReq_pHttpResp) (HttpRequest*, HttpResponse*);

class PageSource {

private:

	// Name of page source
	char* m_pszName;

	// Default error files
	char* m_psz401File;
	char* m_psz403File;
	char* m_psz404File;
	char* m_psz500File;
	char* m_psz501File;

	// Configuration data
	bool m_bBrowsingAllowed;
	bool m_bDefaultFile;
	bool m_bBasicAuthentication;
	bool m_bOverrideGet;
	bool m_bOverridePost;
	bool m_bUseSSI;
	bool m_bUsePageSource;
	bool m_bUseLogging;
	bool m_bUseDailyLogs;

	char* m_pszDefaultFile;
	char* m_pszBasePath;

	// Dynamically linked methods
	Fxn_Int m_pfxnOnInitialize;
	Fxn_Int_pChar_pChar m_pfxnOnBasicAuthenticate;
	Fxn_Int_pHttpReq_pHttpResp m_pfxnOnGet;
	Fxn_Int_pHttpReq_pHttpResp m_pfxnOnPost;
	Fxn_Int m_pfxnOnFinalize;

	HINSTANCE m_hPageSource;

public:
	PageSource();
	virtual ~PageSource();
	
	// Configure
	virtual inline void SetName (char* pszName);
	virtual int LoadLibrary (char* pszLibraryName, char* pszNameSpace);
	virtual inline void SetBasePath (char* pszBasePath);
	virtual inline void SetDefaultFile (char* pszBasePath);
	virtual inline void Set401File (char* psz401File);
	virtual inline void Set403File (char* psz403File);
	virtual inline void Set404File (char* psz404File);
	virtual inline void Set500File (char* psz500File);
	virtual inline void Set501File (char* psz501File);

	virtual inline void SetAllowDirectoryBrowsing (bool bAllowDirectoryBrowsing);
	virtual inline void SetUseDefaultFile (bool bUseDefaultFile);
	virtual inline void SetUseBasicAuthentication (bool bUseBasicAuthentication);
	virtual inline void SetOverrideGet (bool bOverrideGet);
	virtual inline void SetOverridePost (bool bOverridePost);
	virtual inline void SetUseSSI (bool bUseSystemServices);
	virtual inline void SetUsePageSource (bool bUsePageSource);
	virtual inline void SetUseLogging (bool bUseLogging);
	virtual inline void SetUseDailyLogs (bool bUseDailyLogs);

	// Configuration data
	virtual inline char* GetName();
	virtual inline char* GetBasePath();
	virtual inline char* GetDefaultFile();
	virtual inline char* Get401File();
	virtual inline char* Get403File();
	virtual inline char* Get404File();
	virtual inline char* Get500File();
	virtual inline char* Get501File();

	virtual inline bool AllowDirectoryBrowsing();
	virtual inline bool UseDefaultFile();
	virtual inline bool UseBasicAuthentication();
	virtual inline bool OverrideGet();
	virtual inline bool OverridePost();
	virtual inline bool UsePageSource();
	virtual inline bool UseSSI();
	virtual inline bool UseLogging();
	virtual inline bool UseDailyLogs();

	// Methods
	virtual int OnBasicAuthenticate (char* pszLogin, char* pszPassword);
	virtual int OnGet (HttpRequest* pHttpRequest, HttpResponse* pHttpResponse);
	virtual int OnPost (HttpRequest* pHttpRequest, HttpResponse* pHttpResponse);
};

#endif // !defined(AFX_PAGESOURCE_H__1F955CC5_AD9D_11D1_9C61_0060083E8062__INCLUDED_)