// ServerSocket.cpp: implementation of the ServerSocket class.
//
//////////////////////////////////////////////////////////////////////

#include "ServerSocket.h"
#include "File.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

ServerSocket::ServerSocket() {

	// Initialize input buffer parameters
	m_iLen = 0;
	m_iPos = 1;

	// Set socket to invalid
	m_Socket = INVALID_SOCKET;

	// Set port to invalid
	m_siOurPort = INVALID_PORT;

	// Set addresses to be all zeros
	memset ((void*) &m_saOurAddr, '\0', sizeof (m_saOurAddr));
	memset ((void*) &m_saTheirAddr, '\0', sizeof (m_saTheirAddr));

	// Initialize strings
	m_pszTheirIP = new char [NET_ADDRESS_SIZE];
	m_pszTheirHostName = new char [NET_ADDRESS_SIZE];
	m_pszBuffer = new char [MAX_REQUEST_LENGTH + 1];
	m_pszOutputBuffer = new char [MAX_REQUEST_LENGTH + 1];
}

ServerSocket::~ServerSocket() {

	if (m_pszBuffer != NULL) {
		delete [] m_pszBuffer;
	}
	if (m_pszOutputBuffer != NULL) {
		delete [] m_pszOutputBuffer;
	}
	if (m_pszTheirIP != NULL) {
		delete [] m_pszTheirIP;
	}
	if (m_pszTheirHostName != NULL) {
		delete [] m_pszTheirHostName;
	}
}

int ServerSocket::Open() {

	// Open socket
	m_Socket = socket (AF_INET, SOCK_STREAM, 0);

	if (m_Socket == INVALID_SOCKET) {
		return this->GetLastError();
	}

	return OK;
}


int ServerSocket::Listen (short siPort) {

	// Set SO_REUSEADDR to 1
	int iOne = 1;
	setsockopt (m_Socket, SOL_SOCKET, SO_REUSEADDR, (char*) &iOne, sizeof (int));

	// Set our port number
	m_siOurPort = siPort;

	// Configure our address
	m_saOurAddr.sin_family = AF_INET;
	m_saOurAddr.sin_port = htons (m_siOurPort);
	m_saOurAddr.sin_addr.s_addr = INADDR_ANY;

	// Bind to address
	if (bind (m_Socket, (struct sockaddr*) &m_saOurAddr, sizeof (m_saOurAddr)) == SOCKET_ERROR) {
		return this->GetLastError();
	}

	// Begin listening
	if (listen (m_Socket, SOMAXCONN) == SOCKET_ERROR) {
		return this->GetLastError();
	}

	return OK;
}


ServerSocket* ServerSocket::Accept() {

	// Create a new ServerSocket for the incoming connection
	ServerSocket* pConnectedSocket = new ServerSocket();

	// Accept the connection
	int iAddrLen = sizeof (pConnectedSocket->m_saTheirAddr);
	if ((pConnectedSocket->m_Socket = accept (m_Socket, (struct sockaddr*) &(pConnectedSocket->m_saTheirAddr),
		&iAddrLen)) == INVALID_SOCKET) {
		delete pConnectedSocket;
		return NULL;
	}

	// Get their IP address
	sprintf (pConnectedSocket->m_pszTheirIP, inet_ntoa (pConnectedSocket->m_saTheirAddr.sin_addr));

	// Get their hostname
	struct hostent* heName = gethostbyaddr ((char*) &(pConnectedSocket->m_saTheirAddr.sin_addr), sizeof (struct in_addr), 
		AF_INET);

	if (heName != NULL) {
		sprintf (pConnectedSocket->m_pszTheirHostName, heName->h_name);
	} else {
		sprintf (pConnectedSocket->m_pszTheirHostName, pConnectedSocket->m_pszTheirIP);
	}

	return pConnectedSocket;
}


int ServerSocket::RecvRequest() {

	// Put the HTTP request in the buffer
	int iNumBytes = recv (m_Socket, m_pszBuffer, MAX_REQUEST_LENGTH, 0);

	// Set initial position and length
	m_iPos = 1;
	m_iLen = iNumBytes;

/*	if (iNumBytes > 0) {
		char* pszPrint = new char [iNumBytes + 1];
		strncpy (pszPrint, m_pszBuffer, iNumBytes);
		pszPrint [iNumBytes] = '\0';
		printf ("\n***********************************\n");
		printf (pszPrint);
		printf ("\n*************************************\n");
		delete [] pszPrint;
	}
*/
	// Return number of bytes received
	return m_iLen;
}

int ServerSocket::Send (char* pszData) {

	if (int iErrCode = send (m_Socket, pszData, strlen (pszData), 0) == SOCKET_ERROR) {
		return this->GetLastError();
	}

	return OK;
}

int ServerSocket::Send (char* pszData, int iNumBytes) {
	
	if (int iErrCode = send (m_Socket, pszData, iNumBytes, 0) == SOCKET_ERROR) {
		return this->GetLastError();
	}

	return OK;
}

char* ServerSocket::GetNextLine() {

	// Null string
	if (m_iLen == 0 || m_iPos >= m_iLen) {
		return NULL;
	}

	int i, iAdd = 2;
	
	// Search for next "\n" or "/r/n" (telnet EOL) in buffer
	for (i = m_iPos; i < m_iLen; i ++) {
		
		// Check for telnet EOL
		if (m_pszBuffer[i - 1] == '\n') {
			iAdd = 1;
			break;
		}
		else if (m_pszBuffer[i - 1] == '\r' && m_pszBuffer[i] == '\n') {
			iAdd = 2;
			break;
		}
	}

	// Copy string
	int iLength, iBegin = m_iPos - 1;

	if (i == m_iLen) {
		iLength = i - m_iPos + 1;
	} else {
		iLength = i - m_iPos;
	}

	strncpy (m_pszOutputBuffer, &m_pszBuffer[iBegin], iLength);

	// Cap new string
	m_pszOutputBuffer[iLength] = '\0';

	// Reset position to second character after telnet EOF
	m_iPos = i + iAdd;

	return m_pszOutputBuffer;
}


char* ServerSocket::GetNextLine (int* piNumBytes) {

	// Null string
	if (m_iLen == 0 || m_iPos >= m_iLen) {
		*piNumBytes = 0;
		return NULL;
	}

	// Search for next "/r/n" (telnet EOL) in buffer
	for (int i = m_iPos; i < m_iLen; i ++) {
		
		// Check for telnet EOL
		if (m_pszBuffer[i - 1] == '\r' && m_pszBuffer[i] == '\n') {
			break;
		}
	}

	// Copy string
	int iLength, iBegin = m_iPos - 1;

	if (i == m_iLen) {
		iLength = i - m_iPos + 1;
	} else {
		iLength = i - m_iPos;
	}

	strncpy (m_pszOutputBuffer, &m_pszBuffer[iBegin], iLength);

	// Cap new string
	m_pszOutputBuffer[iLength] = '\0';

	// Reset position to second character after telnet EOF
	m_iPos = i + 2;

	*piNumBytes = iLength;

	return m_pszOutputBuffer;
}

char* ServerSocket::GetData() {
	return m_pszBuffer;
}

char* ServerSocket::GetRemainingData() {
	return &m_pszBuffer[m_iPos - 1];
}

int ServerSocket::GetNumRemainingBytes() {
	return m_iLen - m_iPos + 1;
}

int ServerSocket::GetNumBytes() {
	return m_iLen;
}

void ServerSocket::SetPosition (int iPos) {
	m_iPos = iPos - (int) m_pszBuffer;
}

char* ServerSocket::GetClientIP() {
	return m_pszTheirIP;
}

char* ServerSocket::GetClientDomainName() {
	return m_pszTheirHostName;
}


int ServerSocket::Close() {

	// Close the socket
	int iErrorCode = closesocket (m_Socket);

	// Set socket to invalid
	m_Socket = INVALID_SOCKET;

	// Set port to invalid
	m_siOurPort = INVALID_PORT;

	// Set our address to be all zeros
	memset ((void*) &m_saOurAddr, '\0', sizeof (m_saOurAddr));

	// Set their address to be all zeros
	memset ((void*) &m_saTheirAddr, '\0', sizeof (m_saTheirAddr));

	if (iErrorCode == SOCKET_ERROR) {
		return this->GetLastError();
	}

	return OK;
}


bool ServerSocket::Alive() {
	return (m_Socket != INVALID_SOCKET);
}

int ServerSocket::GetLastError() {
	return WSAGetLastError();
}