// Thread.cpp: implementation of the Thread class.
//
//////////////////////////////////////////////////////////////////////

#include "Thread.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

Thread::Thread () {
	m_hThread = NULL;
	m_dwThreadID = -1;
}

Thread::~Thread() {
	::CloseHandle (m_hThread);
}

int Thread::Start (void* pFunctionName, void* pData) {

	if (m_hThread != NULL) {
		::CloseHandle (m_hThread);
	}
	
	m_hThread = ::CreateThread (NULL, 0, (LPTHREAD_START_ROUTINE) pFunctionName, (LPVOID) pData, 
		0, &m_dwThreadID);
	
	return (m_hThread != NULL) ? OK:ERR;
}

int Thread::Start (void* pFunctionName, void* pData, int iPriority) {

	if (m_hThread != NULL) {
		::CloseHandle (m_hThread);
	}

	m_hThread = ::CreateThread (NULL, 0, (LPTHREAD_START_ROUTINE) pFunctionName, (LPVOID) pData, 
		CREATE_SUSPENDED, &m_dwThreadID);

	if (m_hThread == NULL) {
		return ERR;
	}

	if (::SetThreadPriority (m_hThread, iPriority) == 0) {

		// Wrong priority
		::TerminateThread (m_hThread, -1);
		::CloseHandle (m_hThread);
		m_hThread = NULL;

		return ERR;
	}

	return (::ResumeThread (m_hThread) != 0xFFFFFFFF) ? OK:ERR;
}

void Thread::WaitForSingleObject() {
	::WaitForSingleObject (m_hThread, INFINITE);
}


void Thread::Suspend() {
	::SuspendThread (m_hThread);
}

void Thread::Resume() {
	::ResumeThread (m_hThread);
}