#ifndef constants_h_
#define constants_h_

#include <stdio.h>
#include <string.h>
#include <io.h>
#include <fstream.h>
#include <time.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <crtdbg.h>
#include <math.h>
#include <windows.h>

// Error codes
#define INVALID_THREAD -10
#define INVALID_PORT -20
#define ERR -1
#define OK 0

// Buffer sizes
#define MAX_REQUEST_LENGTH 16384
#define NET_ADDRESS_SIZE 128

#define MAX_METHOD_LENGTH 10
#define MAX_VERSION_LENGTH 10

// Mutexes
#define ONE_SECOND 1000	// One second in milliseconds

// HTTP versions
#define UNSUPPORTED_HTTP_VERSION -100
#define HTTP09 0
#define HTTP10 1
#define HTTP11 2

// HTTP methods
#define UNSUPPORTED_HTTP_METHOD -200
#define GET 0
#define POST 1
#define PUT 2
#define HEAD 3
#define TRACE 4

// Form types
#define UNSUPPORTED_FORM -300
#define SIMPLE_FORM 0
#define FILE_FORM 1

// Files
#define MAX_FILENAME_LENGTH 1024
#define MAX_FILE_SIZE_BYTES 1048576		// 1 MB
#define MAX_MIME_TYPE_LENGTH 512
#define AVERAGE_FILE_LENGTH 10240
#define FILE_NOT_FOUND -1
#define FILE_IS_DIRECTORY 0
#define FILE_IS_FILE 1

// Response types
#define RESPONSE_BUFFER 0
#define RESPONSE_FILE 1

// Cookies
#define MAX_COOKIE_SIZE 4096

// Functions
namespace Utility {

	int HexToDec (char szDigit);
	char* WWWEscapeCharToAscii (char* pszInput);
	char* SlashToWhack (char* pszInput);
	char* SlashToUnderscore (char* pszInput);
	char* GetMimeType (char* pszFileName);
	void Sleep (int iMilliSeconds);
	long GetTime();
	char* GetTimeStringGMT();
	char* GetTimeStringLocal();
	char* GetDateStringGMT();
	char* GetDateStringLocal();
	char* GetDateString (SYSTEMTIME* pstTime);
	char* GetDayOfWeekString();
	int GetFileAttributes (char* pszFileName);
	bool ModifiedSince (char* pszFileName, char* pszGMTDate, char** ppszLastModified);
	bool DoesFileExist (char* pszFileName);
	wchar_t* CharToWChar (char* pszString);
	char* WCharToChar (wchar_t* pwszString);
	int AppendString (char** ppszBuffer, char* pszNewString, int iTotalLength);
	void DecodeBase64 (char* pszBase64, char** ppszAnsi);
	char* GetOSVersion();
}

#endif