// Implementation.cpp: implementation of the Implementation class.
//
//////////////////////////////////////////////////////////////////////

#include "Implementation.h"
#include "..\SC\ChatData.h"
#include "..\Alajar\HashTable.h"
#include "..\Alajar\Utility.h"

// ===================
// Response interface:
// ===================
//
// void SetOverride (bool bOverride);
// void SetType (int iType);
//			0 RESPONSE_BUFFER
//			1 RESPONSE_FILE
//
// void SetFileName (char* pszFileName);
// void SetMimeType (char* pszMimeType);
// void SetRedirectUri (char* pszRedirectUri);
// void SetStatus (int iStatus);
//			200 OK
//			301 Moved Permanently (redirect)
//			403 Forbidden
//			404 Not Found
//			500 Internal Server Error
//
// void SetData (char* pszData, int iLength);
// void SetData (char* pszData);
// void ClearData();
// void WriteData (char* pszData);
// void WriteData (char* pszData, int iLength);
//
// void SetCookie (char* pszCookieName, char* pszCookieValue, int iTTLinSeconds, char* pszCookiePath);
// void DeleteCookie (char* pszCookieName, char* pszCookiePath);
//
//
// ==================
// Request interface:
// ==================
//
// int GetMethod();
// int GetVersion();
// char* GetUri();
// char* GetFileName();
// char* GetBrowserName();
// char* GetClientIP();
// char* GetClientDomainName();
//
// int GetNumForms();
// int* GetFormTypes();
// char** GetFormNames();
// char** GetFormValues();
// char** GetFormFileNames();
//	
// int GetNumCookies();
// char** GetCookieNames();
// char** GetCookieValues();
//
// char* GetLogin();
// char* GetPassword();

ChatData* pChatData = NULL;
HashTable<int>* pProfList = NULL;
char* ppszProf [7];

int ChatOnInitialize() {
	
	// Create a new chatroom
	pChatData = new ChatData();

	// Create a new password list
	pProfList = new HashTable<int> (10);

	int* piLand = new int[1]; piLand[0] = 0;
	int* piToueg = new int[1]; piToueg[0] = 1;
	int* piBirman = new int[1]; piBirman[0] = 2;
	int* piKozen = new int[1]; piKozen[0] = 3;
	int* piSelman = new int[1]; piSelman[0] = 4;
	int* piHartmanis = new int[1]; piHartmanis[0] = 5;
	int* piVanLoan = new int[1]; piVanLoan[0] = 6;

	ppszProf[0] = "Land";
	ppszProf[1] = "Toueg";
	ppszProf[2] = "Birman";
	ppszProf[3] = "Kozen";
	ppszProf[4] = "Selman";
	ppszProf[5] = "Hartmanis";
	ppszProf[6] = "Van Loan";

	pProfList->AddElement (ppszProf[0], piLand);
	pProfList->AddElement (ppszProf[1], piToueg);
	pProfList->AddElement (ppszProf[2], piBirman);
	pProfList->AddElement (ppszProf[3], piKozen);
	pProfList->AddElement (ppszProf[4], piSelman);
	pProfList->AddElement (ppszProf[5], piHartmanis);
	pProfList->AddElement (ppszProf[6], piVanLoan);

	return OK;
}

int ChatOnBasicAuthenticate (char* pszLogin, char* pszPassword) {
	
	int* piKey = pProfList->GetElement (pszLogin);

	if (piKey != NULL && strcmp (pszPassword, "") == 0) {
		return OK;
	} else {
		return ERR;
	}
}

int ChatOnGet (HttpRequest* pHttpRequest, HttpResponse* pHttpResponse) {

/*	if (stricmp (pHttpRequest->GetUri(), "/Chat/") == 0) {

		pHttpResponse->SetOverride (true);
		pHttpResponse->SetType (RESPONSE_BUFFER);
		pHttpResponse->SetMimeType (TEXT_HTML);
		pHttpResponse->SetStatus (200);

		RenderChatRoom (pHttpRequest, pHttpResponse);

	} else {
	*/
		pHttpResponse->SetOverride (false);
	//}

	return OK;
}

int ChatOnPost (HttpRequest* pHttpRequest, HttpResponse* pHttpResponse) {

	// Parse through forms
	int i, j, iNumForms = pHttpRequest->GetNumForms();

	char** ppszFormName = pHttpRequest->GetFormNames();
	char** ppszFormValue = pHttpRequest->GetFormValues();

	char* pszProfName = pHttpRequest->GetLogin();
	int* piKey = pProfList->GetElement (pszProfName);

	char* pszTimeString = Utility::GetTimeStringLocal();
	char* pszDateString = Utility::GetDateStringLocal();

	VARIANT vDate, vTime, vKey, vText;
	::VariantInit (&vDate);
	::VariantInit (&vTime);
	::VariantInit (&vKey);
	::VariantInit (&vText);
	vDate.vt = VT_BSTR;
	vDate.bstrVal = TCHARToBSTR (pszDateString);
	vTime.vt = VT_BSTR;
	vTime.bstrVal = TCHARToBSTR (pszTimeString);
	vKey.vt = VT_I4;
	vKey.lVal = piKey[0];
	vText.vt = VT_BSTR;
	
	// Read forms
	bool bRender = true;

	if (iNumForms == 1 && strcmp (ppszFormName[0], "speech") == 0 && ppszFormValue[0] != NULL) {
		vText.bstrVal = TCHARToBSTR (ppszFormValue[0]);
		pChatData->PostMessage (vDate, vTime, vKey, vText);
		::SysFreeString (vText.bstrVal);
	
	} else {
		
		for (i = 0; i < iNumForms; i ++) {
			if (strcmp (ppszFormName[i], "leave.x") == 0) {
				pChatData->ExitChatroom (vDate, vTime, vKey);
				bRender = false;

			}
			
			if (strcmp (ppszFormName[i], "textsubmit.x") == 0) {
				
				for (j = 0; j < iNumForms; j ++) {
					if (strcmp (ppszFormName[j], "speech") == 0) {
						if (ppszFormValue[j] != NULL) {
							vText.bstrVal = TCHARToBSTR (ppszFormValue[j]);
							pChatData->PostMessage (vDate, vTime, vKey, vText);
							::SysFreeString (vText.bstrVal);
						}
					}
				}
			}
		}
	}

	if (bRender) {
		pHttpResponse->SetOverride (true);
		pHttpResponse->SetType (RESPONSE_BUFFER);
		pHttpResponse->SetMimeType (TEXT_HTML);
		pHttpResponse->SetStatus (200);

		RenderChatRoom (pHttpRequest, pHttpResponse);

	} else {
	
		pHttpResponse->SetOverride (true);
		pHttpResponse->SetStatus (301);
		pHttpResponse->SetRedirectUri ("/Chat/");
	}

	delete [] pszTimeString;
	delete [] pszDateString;
	::SysFreeString (vTime.bstrVal);
	::SysFreeString (vDate.bstrVal);

	return OK;
}

int ChatOnFinalize() {

	delete pChatData;
	delete pProfList;

	return OK;
}

void RenderChatRoom (HttpRequest* pHttpRequest, HttpResponse* pHttpResponse) {

	char* pszTimeString = Utility::GetTimeStringLocal();
	char* pszDateString = Utility::GetDateStringLocal();

	pHttpResponse->WriteData ("<html>\n<meta http-equiv=\"refresh\" content=\"60 url=/Chat\">\n<head>\n<title>Official Cornell CS Professor ChatRoom</title>\n</head>\n<body background=\"background.jpg\" text=\"#ffff00\">\n");
	pHttpResponse->WriteData ("<center><h1>CS Professor ChatRoom:</h1>\n<form method = \"post\"\n");

	// Enter chatroom
	char* pszProfName = pHttpRequest->GetLogin();
	int* piKey = pProfList->GetElement (pszProfName);

	VARIANT vResult, vDate, vTime, vKey, vProf;
	::VariantInit (&vDate);
	::VariantInit (&vTime);
	::VariantInit (&vProf);
	::VariantInit (&vKey);
	vDate.vt = VT_BSTR;
	vDate.bstrVal = TCHARToBSTR (pszDateString);
	vTime.vt = VT_BSTR;
	vTime.bstrVal = TCHARToBSTR (pszTimeString);
	vProf.vt = VT_BSTR;
	vProf.bstrVal = TCHARToBSTR (pszProfName);
	vKey.vt = VT_I4;
	vKey.lVal = piKey[0];
	pChatData->EnterChatroom (vDate, vTime, vProf, vKey, &vResult);
	::SysFreeString (vProf.bstrVal);
	::SysFreeString (vTime.bstrVal);
	::SysFreeString (vDate.bstrVal);

	if (vResult.lVal == 0) {
		pHttpResponse->WriteData ("\n<p><strong>There are too many professors in the ChatRoom. Try again later.</strong>");
		pHttpResponse->WriteData ("\n</center>\n</body>\n</html>");
	}

	pHttpResponse->WriteData ("<p><table border = \"3\" cellspacing = \"0\" cellpadding = \"0\" width=\"90%\">\n<tr>\n<td valign = \"top\" bgcolor = \"#331111\">\n<table border = \"0\" cellspacing = \"0\" cellpadding = \"2\" width = \"100%\">\n<tr>\n<td align = \"left\" bgcolor = \"#882266\"><strong>Time</strong></td>\n<td align = \"left\" bgcolor = \"#882266\"><strong>Professor</strong></td>\n<td align = \"center\" bgcolor = \"#882266\"><strong>Message</strong></td>\n</tr>\n");

	// Get number of messages posted
	pChatData->GetNumMessages (&vResult);

	// Read the messages posted
	if (vResult.lVal > 12) {
		vResult.lVal = 12;
	}

	int i;
	VARIANT* pvDate = new VARIANT [vResult.lVal];	char* pszDate;
	VARIANT* pvTime = new VARIANT [vResult.lVal];
	VARIANT* pvKey = new VARIANT [vResult.lVal];
	VARIANT* pvMessage = new VARIANT [vResult.lVal]; char* pszMessage;
	pChatData->ReadMessages (vResult.lVal, pvDate, pvTime, pvKey, pvMessage, &vResult);

	for (i = 0; i < vResult.lVal; i ++) {
		pHttpResponse->WriteData ("<tr>\n<td align = \"left\" width = \"30%\" valign = \"top\" bgcolor = \"#331111\">\n<font size = \"2\" color = \"#ffff00\">");
		
		// Date
		pszDate = BSTRToTCHAR (pvDate[i].bstrVal);
		pHttpResponse->WriteData (pszDate);
		delete [] pszDate;
		::SysFreeString (pvDate[i].bstrVal);
		::SysFreeString (pvTime[i].bstrVal);
		
		pHttpResponse->WriteData ("</font></td>\n<td align = \"left\" width = \"5%\" valign = \"top\" bgcolor = \"#331111\">\n<strong>");

		if (pvKey[i].lVal > -1) {
			pHttpResponse->WriteData ("<font color = \"#ffff00\">&lt;");
			pHttpResponse->WriteData (ppszProf[pvKey[i].lVal]);
		} else {
			pHttpResponse->WriteData ("<font color = \"#ffffff\">&ltAdmin");
		}

		pHttpResponse->WriteData ("&gt;</font></strong><font color = \"#ffffaa\"></td><td>");
	
		pszMessage = BSTRToTCHAR (pvMessage[i].bstrVal);
		pHttpResponse->WriteData (pszMessage);
		delete [] pszMessage;
		::SysFreeString (pvMessage[i].bstrVal);
			
		pHttpResponse->WriteData ("</font></td>\n</tr>\n");
	}

	delete [] pvDate;
	delete [] pvTime;
	delete [] pvKey;
	delete [] pvMessage;
	
	pHttpResponse->WriteData ("</table>\n</td>\n<td valign = \"top\" bgcolor = \"#331111\" width = \"20%\">\n<table border = \"0\" cellspacing = \"0\" cellpadding = \"2\" width = \"100%\">\n<tr>\n<td align = \"center\" width = \"3%\" bgcolor = \"#882266\"><strong>Professors</strong></td>\n</tr>\n<tr>\n<td valign = \"top\" bgcolor = \"#331111\">\n");

	pChatData->GetNumEmpires (&vResult);

	VARIANT* pvProfKey = new VARIANT [vResult.lVal];
	
	VARIANT vNumProfs;
	pChatData->GetEmpireKeys (vResult.lVal, pvProfKey, &vNumProfs);
	
	for (i = 0; i < vNumProfs.lVal; i ++) {
		pHttpResponse->WriteData (ppszProf [pvProfKey[i].lVal]);
		pHttpResponse->WriteData ("<br>");
	}

	delete [] pvProfKey;

	pHttpResponse->WriteData ("</tr>\n</table>\n</td>\n</tr>\n</table>\n<p><input type=\"text\" size=\"75%\" name=\"speech\">&nbsp<input type = \"image\" border = \"0\"\nsrc = \"Speak.gif\" name = \"textsubmit\" value = \"Speak\">\n<p><input type = \"image\" border = \"0\" src = \"LeaveTheChatroom.gif\" name = \"leave\">\n<input type = \"image\" border = \"0\" src = \"RefreshMessages.gif\" name = \"refresh\">\n</center></form></body></html>");

	delete [] pszTimeString;
	delete [] pszDateString;
}